function varargout = magenmpc_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'magenmpc'};

    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        p.online.path = 'online'; p.online.shortid = 'onl';
        p.comm.path = 'comm';
        p.X.path = 'X'; p.X.type = 'vector';
        p.trials = []; p.maxsteps = []; 
        p.reset.path = 'reset'; p.reset.type = 'array'; p.reset.shortid = 'res';
        % planning algorithm and parameters
        p.planalg.path = 'planalg'; p.planalg.shortid = '';
        % generic parameters
        p.planparamsn.path = 'planparams.n'; p.planparamsn.shortid = 'n'; p.planparamsn.label = 'budget'; % p.planparamsn.scale = 'log';
        % U is tougher to handle as it can be a cell array of cell arrays (the action spaces for each agent)...
        % the value is formatted to the number of actions of the first agent
        p.planparamsU.path = 'planparams.U'; p.planparamsU.type = 'cell'; 
        p.planparamsU.shortid = 'nU'; p.planparamsU.label = 'number of discrete actions';
        p.planparamsU.formatfun = @(val) sprintf('%d', length(val{1}));
        % OP-specific params
        p.planparamscriterion.path = 'planparams.criterion'; p.planparamscriterion.shortid = '';
        p.planparamsrob.path = 'planparams.rob'; p.planparamsrob.shortid = 'rob';
        p.planparamsxpredtype.path = 'planparams.xpredtype'; p.planparamsxpredtype.shortid = 'xpt';
        p.planparamsmaxpredlength.path = 'planparams.maxpredlength'; p.planparamsmaxpredlength.shortid = 'predlen';
        p.planparamsmaxpredlength.label = 'length of communicated predictions';
        p.planparamsmaxpredlength.formatfun = @formatpredlength;    % this is to deal with Inf values
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'model_params', 'reset', 'datadir', 'verb', 'sampledisp', 'trialdisp', 'repeat');
        varargout = {p};
        
%     % provide input variables along which evalbatch will then be able to plot in "inner" mode
%     case 'inputs',
%         in = struct;
%         in.planparamsn.type = 'scalar';
%         in.planparamsn.id = 'planparamsn';
%         in.planparamsn.label = 'n';
%         in.planparamsn.val = [];
% 
%         in.planparamsmem_maxsize.type = 'scalar';
%         in.planparamsmem_maxsize.id = 'planparamsmem_maxsize';
%         in.planparamsmem_maxsize.label = 'memsize_{max}';
%         in.planparamsmem_maxsize.val = [];
% 
%         varargout = {in};
        
    % provide output variables
    case 'outputs',
        o = struct;

        % disagreement between agent velocities
        o.disagr.id = 'disagr';
        o.disagr.label = 'disagreement';
        o.disagr.params = {'steps', 'avgdisagr'};

        % graph DISconnectivity
        o.disconn.id = 'disconn';
        o.disconn.label = 'disconnectivity';
        o.disconn.params = {'steps'};
        % should only be plotted as markers
        o.disconn.meansty = {{'-', 'Color', 'r', 'LineStyle', 'none', ...
            'Marker', 'X', 'MarkerSize', 10, 'LineWidth', 2.5}};

        % total execution time
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';
        
%         % average planning time per state; for online: the same as the planning time/step
%         o.plantime.id = 'plantime';
%         o.plantime.label = 'planning time';
%         o.plantime.units = 's';
%         o.plantime.scale = 'log';
% 
%         % average tree depth
%         o.depth.id = 'depth';
%         o.depth.label = 'tree depth';
        
        varargout = {o};
        
    % process options
    case {'run'},
        cfg.(mode) = 1;
        % disp('Dummy call magenmpc with config:'); dispx(cfg, 0, 0, 'cfg', 'config'); dispx(cfg.planparams, 0, 0, 'planparams', 'config');
        magenmpc(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.run};
        
%     % compute average planning time per state
%     case 'plantime',
%         load(cfg.datafile, 'timestat');
%         % make column vector for easy handling
%         plantime = timestat.planning(:);
%         % note that, for online case, there may be noninitialized (nan) planning times; ignore them
%         varargout = {mean(plantime(~isnan(plantime)))};
% 
%     % find average tree depth
%     case 'depth',
%         load(cfg.datafile, 'planinfoh');
%         varargout = {mean([planinfoh(:).depth])};

    % evaluate disagreement
    case 'disagr',
        % load experiment config
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'X', 'model'});
        % error checks
        if ~e_cfg.online, error('[disagr]: online solution required'); 
        elseif ~(strcmp(e_model.id, 'malinear') || strcmp(e_model.id, 'manonhol')) ...
                || ~strcmp(e_model.rewtype, 'flock'), 
            error('[disagr]: model must be linear or nonholonomic, reward flocking'); 
        end;
        n = e_model.nagents;
        if isempty(cfg.steps),  steps = 1:e_cfg.maxsteps+1;
        else                    steps = cfg.steps;
        end;
        % compute disagreement vector for each trial and step
        if isfield(e_model, 'dim'), agree = e_model.dim+1:e_model.p(1);
        else                        agree = e_model.agree;
        end;
        disagr = zeros(length(steps), e_cfg.trials);
        for trial = 1:e_cfg.trials,
            x = reshape(e_X(:, :, trial), e_model.p(1), n, e_cfg.maxsteps+1);
            xa = x(agree, :, :);
            % compute sum of distances explicitly using FORs, for clarity
            for is = 1:length(steps),
                s = steps(is);
                for i = 1:n,
                    for j = i+1:n,
                        disagr(is, trial) = disagr(is, trial) + sqrt(sum((xa(:, i, s) - xa(:, j, s)) .^ 2));
                    end;
                end;
                if isfield(cfg, 'avgdisagr') && cfg.avgdisagr,
                    disagr(is, trial) = disagr(is, trial) / (n*(n-1)/2);
                end;
            end;
        end;
        varargout = {mean(disagr, 1), disagr};

    % evaluate disconnectivity: data assumed superimposed on the disagreement plot
    % so each graph that is DISconnected will get value Y where Y = disagreement
    % each connected graph will get NaN (so it does not appear on the final graph)
    case 'disconn',
        % load experiment config
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'GRAPHS', 'model'});
        % error checks
        if ~e_cfg.online, error('[conn]: online solution required'); 
        elseif ~strcmp(e_model.id, 'malinear'), error('[conn]: unsupported model type'); 
        end;
        % first, recompute the disagreements to have proper y data
        disagr = magenmpc_exp('disagr', cfg);
        if isempty(cfg.steps),  steps = 1:e_cfg.maxsteps+1;
        else                    steps = cfg.steps;
        end;
        % examine connectivity for each trial and step
        conn = false(length(steps), e_cfg.trials);
        for trial = 1:e_cfg.trials,
            % compute sum of distances explicitly using FORs, for clarity
            for is = 1:length(steps),
                s = steps(is);
                G = e_GRAPHS(:, :, s, trial);
                lambdas = eig(diag(sum(G)) - G);    % eigenvalues of Laplacian
                % take into account numerical errors
                conn(is, trial) = sum(abs(lambdas) <= 1e-10) <= 1;
            end;
        end;
        % graphconnectedatsteps = conn'
        conn = all(conn, 1);
        disconn(conn) = NaN(1, e_cfg.trials);
        disconn(~conn) = disagr(~conn);
        varargout = {disconn};
        
    otherwise,  % return empty
        varargout = {[]};
end;

end     % main function genmpc_exp


% %%%%%%%% local funs
function val = formatpredlength(val)
if isfinite(val),   val = num2str(val);
else                val = 'Inf'; 
end;
end


%=============== Older code ==============

    % evaluate return w.r.t average disagreement (old variant)
%     case 'disagr',
%         % load experiment config
%         loadwithprefix(cfg.datafile, 'e_', {'cfg', 'X', 'model'});
%         % error checks
%         if ~e_cfg.online, error('[disagr]: online solution required'); 
%         elseif ~strcmp(e_model.id, 'malinear'),
%             error('[disagr]: unsupported model type'); 
%         end;
%         % compute disagreement vectors for each trial and step,
%         % w.r.t. current average of the velocities
%         disagr = nan(e_cfg.maxsteps+1, e_cfg.trials);
%         for trial = 1:e_cfg.trials,
%             x = reshape(e_X(:, :, trial), e_model.p(1), e_model.nagents, e_cfg.maxsteps+1);
%             vel = squeeze(x(e_model.dim+1:end, :, :));
%             disagr(:, trial) = sum((vel - repmat(mean(vel, 1), e_model.nagents, 1)).^2, 1);
%             disagr(:, trial) = sqrt(disagr(:, trial));
%         end;
%         % keep only the last steps if configured
%         if ~isempty(cfg.steps),
%             disagr = disagr(cfg.steps, :);
%         end;
%         varargout = {mean(disagr, 1), disagr};