function [params, configs, used, expanded] = processparams(cfg, allparams, distribute)

params = struct;
configs = {};
used = {};
expanded = {};

% parse parameters, determining which of them are set 
% and which have multiple values (are "expanded")
indices = {};
pids = fieldnames(allparams);
for i = 1:length(pids),
    % retrieve parameter
    pid = pids{i};
    p = allparams.(pid);
    if isempty(p),      p = struct('id', pid, 'path', pid);    % auto set id and path to pid
    elseif ischar(p),   p = struct('id', pid, 'path', p);      % auto set id and path to value
    else
        if ~isfield(p, 'id'), p.id = pid; end;          % set id as pid
        if ~isfield(p, 'path'), p.path = p.id; end;     % path is by default same as id
    end;
    if ~isfield(p, 'type'), p.type = 'generic'; end;    % auto set type

    % find param on config, allowing for any structure depth
	path = p.path; val = cfg; found = 1;
    p.pathel = {};
    while ~isempty(path),
        [id, path] = strtok(path, '.');
        p.pathel{end+1} = id;
        if ~isfield(val, id),   found = 0; break; 
        else                    val = val.(id);
        end;
    end;
    
    % if the parameter was found on the config, add it
    if found,
        % add the param to used parameter values
        p.val = val;
        params.(pid) = p;
        used{end+1} = pid;
        % 'distributed' parameter
        if ~isempty(distribute) && any(strcmp(pid, distribute)),
            indices{end+1} = -1;
        % parameter is expanded if it is not trivial, and it has:
        % several numerical elements, and the basic parameter does NOT have type vector or generic array
        % OR several cells, but the basic parameter does NOT have type cell array or generic array
        % HOWEVER if it is a cell array of cell arrays, then DO consider it expanded (to allow for
        % cell array parameters)
        elseif ~strcmp(p.type, 'trivial') && length(val) > 1 && ...
            ( (isnumeric(val) && ~any(strcmp(p.type, {'vector', 'array'}))) ...
                || (iscell(val) && ~any(strcmp(p.type, {'cell', 'array'}))) ...
                || (iscell(val) && iscell(val{1})) ),
            expanded{end+1} = pid;
            indices{end+1} = 1:length(val);
        else
            indices{end+1} = 0;
        end;
    end;
end;

% form indivdual experiment configs
indices = flat(indices);
nexp = size(indices, 2);
for i = 1:nexp,
    c = struct;
    for j = 1:length(used),
        p = params.(used{j});
        switch indices(j, i)
            case 0,      % single value
                eval(sprintf('c.%s=p.val;', p.path));
            case -1,     % distributed param, one value for each exper
                if isnumeric(p.val),    % numerical array of values
                    eval(sprintf('c.%s=p.val(%d);', p.path, i));
                else                    % (assumed) cell array of values
                    eval(sprintf('c.%s=p.val{%d};', p.path, i));
                end;
            otherwise,   % expanded param
                if isnumeric(p.val),    % numerical array of values
                    eval(sprintf('c.%s=p.val(%d);', p.path, indices(j, i)));
                else                        % (assumed) cell array of values
                    eval(sprintf('c.%s=p.val{%d};', p.path, indices(j, i)));
                end;
        end;
    end;
    configs{i} = c;
end;
