
%% model creation, initialization
modelcfg = struct;
modelcfg.size = [5 5];
modelcfg.x_goal = [5; 5];
modelcfg.x_obst = [2 1; 4 4; 4 5]';
model = gridnav_problem('model', modelcfg);
% init view
viewcfg = struct;
viewcfg.model = model;


%% trajectory simulation with random initial state and actions
x = gridnav_problem('reset', model, 'rand');

% show initial state
viewcfg.x = x;
viewcfg.gview = gridnav_visualize(viewcfg);
terminal = 0;
k = 0; maxsteps = 200;
while ~terminal && k < maxsteps,
    [x, r, terminal] = gridnav_mdp(model, x, unidrnd(4));
    viewcfg.x = x;
    viewcfg.gview = gridnav_visualize(viewcfg);
    k = k + 1;
end;


% %% show a policy that always goes up
% viewcfg.x = [];
% viewcfg.Q = [];
% viewcfg.h = 4 + zeros(model.size);
% viewcfg.gview = gridnav_visualize(viewcfg);
% 
% %% show a random Q-function
% viewcfg.x = [];
% viewcfg.h = [];
% viewcfg.Q = rand([model.size, 4]);
% viewcfg.gview = gridnav_visualize(viewcfg);


%% run Q-iteration, visualize the evolution of the Q-function and policy
% also show optimal Q-function as a surface

qicfg = struct;
qicfg.run = 1;
qicfg.problem = 'gridnav_problem';
qicfg.model_params = {modelcfg};
qicfg.gamma = 0.75;
qicfg.eps = 0.01;
qicfg.datadir = '.';
qicfg.datafile = 'lab1_qisol';
tic;
qiter(qicfg);
toc

load(qicfg.datafile, 'Q', 'Qh', 'model');

ell = 2;
viewcfg = struct;
viewcfg.model = model;
while ~isempty(Qh{ell}),
    Q = reshape(Qh{ell}, model.size(1), model.size(2), 4);
    [Qmax, h] = max(Q, [], 3);
    viewcfg.Q = Q;
    viewcfg.h = h;
    viewcfg.gview = gridnav_visualize(viewcfg);
    viewcfg.pause = -1;
    ell = ell + 1;
end;
 
Q = reshape(Q, model.size(1), model.size(2), 4);
figure;
subplot(221); mesh(Q(:, :, 1)');
subplot(222); mesh(Q(:, :, 2)');
subplot(223); mesh(Q(:, :, 3)');
subplot(224); mesh(Q(:, :, 4)');


%% run policy iteration, visualize the evolution of the Q-function and policy
% also show optimal Q-function as a surface

picfg = struct;
picfg.run = 1;
picfg.problem = 'gridnav_problem';
picfg.model_params = {modelcfg};
picfg.gamma = 0.95;
picfg.eps = 0.001;
picfg.peval_eps = 0.001;
picfg.datadir = '.';
picfg.datafile = 'lab1_pisol';
tic;
piter(picfg);
toc

load(picfg.datafile, 'h', 'Q', 'Qh', 'hh', 'model');

ell = 2;
viewcfg = struct;
viewcfg.model = model;
while ~isempty(Qh{ell}),
    viewcfg.Q = reshape(Qh{ell}, model.size(1), model.size(2), 4);
    viewcfg.h = reshape(hh{ell}, model.size(1), model.size(2));
    viewcfg.gview = gridnav_visualize(viewcfg);
    viewcfg.pause = -1;
    ell = ell + 1;
end;
 
Q = reshape(Q, model.size(1), model.size(2), 4);
figure;
subplot(221); mesh(Q(:, :, 1)');
subplot(222); mesh(Q(:, :, 2)');
subplot(223); mesh(Q(:, :, 3)');
subplot(224); mesh(Q(:, :, 4)');


%% trajectory simulation with the computed state and actions
x = gridnav_problem('reset', model, 'rand');

h = reshape(h, model.size(1), model.size(2));
% show initial state
viewcfg.x = x;
viewcfg.gview = gridnav_visualize(viewcfg);
terminal = 0;
k = 0; maxsteps = 200;
while ~terminal && k < maxsteps,
    [x, r, terminal] = gridnav_mdp(model, x, h(x(1), x(2)));
    viewcfg.x = x;
    viewcfg.gview = gridnav_visualize(viewcfg);
    k = k + 1;
end;