function [c, rad] = gen_tvr_nrbfs(cc, gamma, eps)

if nargin < 3,
    eps = 0.0001;
end;

p = length(cc);     % number of variables
for ip = 1:p,   % size of center grids
    dimc(ip) = length(cc{ip});
end;
N = prod(dimc);      % total number of centers

% compute all neighbor delta indices (transform into matrix, subtract 1 to obtain a range -1, 0, 1
nbdelta = dec2base(0:3^p-1, 3) - '0' - 1;
% we do not consider diagonal neighbors, only direct ones, 
% i.e. ones with p - 1 coordinates == 0
direct = find(sum(nbdelta == 0, 2) == p - 1);
nbdelta = nbdelta(direct, :);
nbn = length(direct);

NB = cell(N, 1);           % neighbours table
NBC = zeros(nbn, 1);       % neighbours count
nbbound = repmat(dimc, nbn, 1);
for i = 1:N,
    nbi = repmat(lin2ndi(i, dimc), nbn, 1) + nbdelta;
    inrange = find(all(nbi > 0 & nbi <= nbbound, 2));
    NB{i} = ndi2lin(nbi(inrange, :), dimc);
    NBC(i) = length(inrange);
end;


c = flat(cc);
rad = 0 * c;

% minimize 1/rad^2, i.e. maximize radii
lp.f = ones(p, 1);
% keeping 1/rad^2 positive, i.e. radius making sense
lp.lb = zeros(p, 1) + eps;
% % and keeping 1/rad^2 at reasonable values
% lp.ub = zeros(p, 1) + 3000;
lp.ub = [];
totalb = (1-gamma) / 2;
% don't display anything (many lp problems will be solved)
lp.opt = optimset('Display', 'off');  
for i = 1:N, 
    % compute constraint matrix D of size (nof neighbours) x p, 
    % other RBFs on rows, state variable index on columns (that's why
    % the diff matrix is transposed)
    lp.D = (repmat(c(:, i), 1, NBC(i)) - c(:, NB{i}))' .^ 2;
    lp.b = - log(totalb ./ NBC(NB{i})) + eps;
    % maximize radii
    [rad(:, i) lplambda lpflag] = linprog(lp.f, -lp.D, -lp.b, [], [], lp.lb, lp.ub, [], lp.opt);
    if lpflag < 0,
        dispx(['LP did not converge! rbf#=' num2str(i) ...
            ' lpflag=' num2str(lpflag) ' 1/rad^2=' num2str(reshape(lp.sol(:, i), 1, []))], cfg.verb, 3);
    end;
end;
rad = 1 ./ sqrt(rad);


% % code to plot RBFs in 2-D
% x = cc{1}(1):.1:cc{1}(end);
% y = cc{2}(1):.1:cc{2}(end);
% phi = zeros(length(x), length(y), N);
% for i = 1:length(x),
%     for j = 1:length(y),
%         phi(i, j, :) = nrbf([x(i); y(j)], N, c, rad);
%     end;
% end;
% figure;
% for i = 1:N,
%     mesh(y, x, squeeze(phi(:, :, i))); hold on;
% end;
