function figh = plotrbfs(cfg)
% Plot RBFs

CFG.plottarget = 'screen';
CFG.savedir = pwd;
CFG.savefig = 'rbfs';
CFG.flatcolor = 0;          % use same surface color for every MF in 2D plot
CFG.placeticks = 1;         % place ticks on centers of MFs or not
CFG.c = [];
CFG.rad = [];
CFG.res = 50;
% allow customization of labels
CFG.xlabels  = {'x_1', 'x_2'};      % variables
CFG.mulabel = '\phi(x)';            % membership degree
CFG.randseed = [];                  % for repeatable colors
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

cfg = parseconfig(cfg, CFG);

if ~isempty(cfg.randseed), rand('twister', cfg.randseed); end;

% plot
figh = figure;

if cfg.flatcolor,   % use flat edge color, as well
    lsty = cell(18, 1);
    for i = 1:length(lsty), lsty{i} = {'k-'}; end;
else
    % TODO need to handle case where color is not flat but plottarget=latex!
    lsty = {{'r-'}, {'b-'}, {'g-'}, {'k-'}, {'y'}, {'m'}, {'c'}, ...
        {'r--'}, {'b--'}, {'g--'}, {'k--'}, {'m--'}, {'c--'}, ...
        {'r:'}, {'b:'}, {'g:'}, {'k:'}, {'m:'}, {'c:'}};
end;

% make sure centers are in flat format
if isempty(cfg.c), error('Centers must be specified');
elseif iscell(cfg.c), cfg.c = flat(cfg.c); 
else    % assume it's flat format and leave as it is
end;
% find out #vars and #RBFs
d = size(cfg.c, 1);
N = size(cfg.c, 2);
% if no radii given, generate automatically as averages of inter-RBF distances
% while assuming the RBFs are distributed on a grid
if isempty(cfg.rad),        
    cgrid = unflat(cfg.c);
    cfg.rad = {};
    for i = 1:d,
        r = diff(cgrid{i}); r = [r(1) r r(end)]; 
        r = (r(1:end-1) + r(2:end)) ./ 2;
        cfg.rad{i} = r;
    end;
    cfg.rad = flat(cfg.rad);
end;

xmin = min(cfg.c, [], 2); xmax = max(cfg.c, [], 2);
xmin
xmax

% enforce any dimensions override
if d == 2,
    if ~cfg.addtocrt, figh(end+1) = figure; 
    else figh(end+1) = gcf; 
    end;
    res = cfg.res;
    X = {xmin(1) : (xmax(1)-xmin(1))/res : xmax(1), ...
        xmin(2) : (xmax(2)-xmin(2))/res : xmax(2)};
    Xf = flat(X);
    Np = size(Xf, 2); 
    Yf = zeros(Np, N);
    for i = 1:Np, Yf(i, :) = nrbf(Xf(:, i), N, cfg.c, cfg.rad); end;
    for j = 1:N,
        if cfg.flatcolor,   col = 0.7 * ones(3, 1);
        else                col = 0.3 + 0.6 * rand(3, 1);
        end;
        mesh(X{1}, X{2}, reshape(Yf(:, j), length(X{1}), length(X{2}))', ...
            'EdgeColor', max([0;0;0], col - 0.35), 'FaceColor', col, 'FaceAlpha', 1); hold on;
    end;

    xlim([xmin(1) xmax(1)]); ylim([xmin(2) xmax(2)]);
    xlabel(cfg.xlabels{1}); ylabel(cfg.xlabels{2})
    zlabel(cfg.mulabel); 
else
    error('Currently only 2 dimensions supported');
end;

setfigprop(cfg);
saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);

