function figh = mahamm_plot(cfg)
% Plot multiagent linear system evolution, with a focus on flocking

% default arguments
if nargin < 1, cfg = ''; end;
% support for default calling mode -- i.e. only history
if isstruct(cfg) && isfield(cfg, 't') && isfield(cfg, 'x'),
    hist = cfg; cfg = struct; cfg.traj = hist;
    cfg.datasource = 'caller';
end;

CFG.problem = 'mahamm_problem';
CFG.datasource = '';
CFG.datafile   = '';    % data source can be given as a data file here as well
CFG.traj = [];          % plot history (given as standard history structure)
% plot configuration
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

% process config
cfg = parseconfig(cfg, CFG);
if ~isempty(cfg.datafile), cfg.datasource = cfg.datafile; end;

% cfg

% needed vars
vars = {'model', 'cfg'};
% load model, grids, etc. from the data source
cfg1 = cfg;
switch cfg.datasource,
    case 'caller',              % load needed vars from caller space
        for i=1:length(vars), cv.(vars{i}) = evalin('caller', vars{i}); end;
        structtovars(cv);
    otherwise                   % load from file
        load(cfg.datasource, vars{:});
end;
expcfg = cfg; cfg = cfg1; clear cfg1; % swap config

% color style
cs = struct;
cs.agentcolors = {'b', 'r', 'g', 'k', 'm', 'y', 'c', [.5 .5 .5], [1 .5 .5]};
% cs.agenttraces = repmat({{'LineStyle', ':'}}, 1, length(cs.agentcolors));
cs.agentlines = repmat({{}}, 1, length(cs.agentcolors));
% cs.agentsize = .3;
% cs.connections = {'Color', [.5 .5 .5], 'LineStyle', '-', 'LineWidth', 0.5};
% cs.velocity = {'LineStyle', '-', 'LineWidth', 2};
% cs.range = {'LineStyle', '--', 'LineWidth', 1};
% cs.return = {'Color', 'g', 'LineWidth', 1.5};
% grayscale style starting from color style
gs = cs;
% choose style
if cfg.grayscale, sty = gs; else sty = cs; end;
% labels
labels = struct;
labels.x = 'x'; labels.u = {'u'}; labels.time = 'k*T_s [s]';


% %%%%%%%%%%%%%%%%%%%%%%
% PLOT DATA

figh = [];

if ~isempty(cfg.traj),
    figh(end+1) = figure;
    traj = cfg.traj;
    % pick up positions, velocities
    x = reshape(traj.x, model.p(1), model.nagents, traj.Ns+1);

    for d = 1:model.p(1),
        subplot(1, model.p(1), d); 
        for i = 1:model.nagents,
            plot(traj.t, squeeze(x(d, i, :)), 'Color', sty.agentcolors{i}, sty.agentlines{i}{:}); 
            hold on;
        end;
        xlabel(labels.time); ylabel(sprintf('%s_%d', labels.x, d));
    end;
end;

% save last figure if indicated
if ~isempty(figh), saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget); end;

end
% END RETURNING array of figure handles =================================================

