function  varargout = mahamm_problem(mode, varargin)
% Markov game with decoupled linear dynamics and homogeneous agents.
%   OUT = MAHAMM_PROBLEM(MODE, CONFIG)
%   [X0, G] = MAHAMM_PROBLEM('reset', MODEL, X0) is the only reset mode supported

switch mode,

    % A short identifier
    case 'shortid', varargout = {'mahamm'};
    
    % The 'model' mode is required
    case 'model',
        % these should stay fixed
        m.id = 'mahamm';                  % an identifier
        m.problem = @mahamm_problem;      % handle to m function
        m.det = 1;                        % whether problem is deterministic or stochastic        
        m.fun = 'mahamm_mg';              % dynamics and rewards function
        m.plotfun = [];
        m.visualizefun = [];
        m.simagent = 'mahamm_simagent';
        
        % configurable
        m.nagents = [];                     % number of agents
        m.matype = 'fixedconn';             % 'varconn' or 'fixedconn' problem
        m.G = [];                           % connectivity graph, for fixed connectivity
        m.Ts = 1;                           % sample time
        m.A = [];                           % transfer matrix
        m.B = [];                           % input matrix
        m.inputnl = [];                     % input nonlinearity
        m.rewtype = 'none';                 % no reward supported
        
        % process any custom config
        if nargin >= 2, m = parseconfig(varargin{1}, m); end;
        
        switch m.matype,
            case 'fixedconn',
            otherwise,
                error('Unknown multiagent type [%s]', m.matype);
        end;
        
        m.p = size(m.A, 1) + zeros(1, m.nagents);
        m.q = size(m.B, 2) + zeros(1, m.nagents);
        m.pranges = [0 cumsum(m.p)]; m.qranges = [0 cumsum(m.q)];   % variable ranges

        switch m.rewtype,
            case 'none',
            otherwise,
                error('Unknown reward type [%s]', m.rewtype);
        end;
        varargout = {m};

    case 'reset',   % handle state reset
        m = varargin{1};
        res = varargin{2};
        if ~isnumeric(res), error('Only direct numerical reset supported'); end;
        x0 = res;
        G = m.G; % return fixed connections graph
        % flatten and output x, output G
        varargout = {x0(:), G};
                
end;        % mode SWITCH

end % =================== MAIN FUNCTION

% END sample_problem() RETURNING out
% ===================================================