function xplus = invpend_discretestep(m, x, u)
% Implements the discrete-time dynamics of an inverted pendulum

% limit torque
u = max(-m.maxu1, min(m.maxu1, u));
% integrate numerically along the sample time
if m.odemethod(1) == 'o',           % ODE
	[odet odey] = feval(m.odesolver, @invpend_trans, m.odet, x, m.odeopt, m, u);
    xplus = odey(end, :)';          % pick up the state at t + Ts
elseif m.odemethod(1) == 'f',       % fixed-step ODE
    odey = feval(m.odesolver, @invpend_trans, m.odet, x, m, u);
    xplus = odey(end, :)';          % pick up the state at t + Ts
elseif m.odemethod(1) == 'e',       % Euler, possibly multistep
    dt = m.Ts / m.odesteps; xplus = x;
    for i = 1:m.odesteps,
        xplus = xplus + dt * invpend_trans(0, x, m, u);
    end;
end;

% normalized state: wrap angles in [-pi, pi), bound velocities
xplus(1) = mod(xplus(1) + pi, 2*pi) - pi;
xplus(2) = max(-m.maxomega, min(m.maxomega, xplus(2)));
end     % ipsetup_singletransition local function

function dx = invpend_trans(t, x, m, u)
dx(1,1) = x(2);
dx(2,1) = (m.mgl*sin(x(1)) - (m.b+m.K^2/m.R)*x(2) + m.K/m.R*u) / m.J;
end
