%% OP experiments for flocking
function manonhol_opflocking(mode, expids, custombcfg, customacfg)

%% General config
% clear all; close all;

mcfg = struct;
mcfg.nagents = 9;
mcfg.range = 5;
mcfg.beta = .5;
mcfg.Ts = 0.25;
dvmax = 5; mcfg.Q = eye(2) ./ dvmax^2;

% initial state
% initial position
gridstep = 3;
gr = zeros(2, 3, 3); 
gr(1, :, :) = [-1 0 1; -1 0 1; -1 0 1];
gr(2, :, :) = [1 1 1; 0 0 0; -1 -1 -1];
p0 = reshape(gr .* gridstep, 2, mcfg.nagents);
% initial state for 2D control
x0 = [p0; 
    1.5 .*[1 1 1 1 0 1 1 1 1];                  % velocity
    pi .* [3/4 1 -3/4 1/2 0 -1/2 1/4 0 -1/4]];  % heading
% initial state for constant-velocity model
x0dubbins = [p0; 
    1.5 .*[1 1 1 1 1 1 1 1 1];                  % velocity
    pi .* [3/4 1 -3/4 1/2 0 -1/2 1/4 0 -1/4]];  % heading

% batch experiment
BCFG = struct;
BCFG.alg = 'magenmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planparamsn', 'planparamsmaxpredlength'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithms
ACFG = struct;
ACFG.problem = 'manonhol_problem';
ACFG.model_params = {mcfg};
ACFG.online = 1;
ACFG.comm = 'seq';
ACFG.gamma = 0.95;
ACFG.planalg = 'maopd_predictneighbors';
        Ugrid = flat({[-0.5 0 0.5], [-pi/3 0 pi/3]});
ACFG.planparams.U = repmat({Ugrid}, 1, mcfg.nagents);
ACFG.planparams.xpredtype = 'flatcomm';   
ACFG.planparams.rob = 0;
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 60;
% visualization
ACFG.verb = 3;
ACFG.datadir = 'd:\Work\m\approxrl\masystems\manonhol\manonhol_opflocking';
ACFG.steps = []; %41:60; 
ACFG.avgdisagr = 1;
ACFG.reset = x0(:);

cd(ACFG.datadir); 


%% Local execution function 
% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'disagr', 'disconn', 'exectime', 'depth'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

    function execsimpleexp(expid, algo, acfg)
        if any(strcmp(expid, expids)),  % if experiment is requested
            acfg = copyfields(customacfg, acfg);
            acfg.(mode) = 1;
            algo(acfg);
        end;
    end

%% Basic experiments: classical control
acfg = ACFG;
    m2cfg = mcfg; m2cfg.Ts = 0.025;
    acfg.model_params = {m2cfg};
    acfg.maxsteps = 15/m2cfg.Ts;
acfg.idinfix = 'classical';
acfg.comm = 'none';
acfg.planalg = 'masimplecontrol';
acfg.planparams.type = 'dubbins';
acfg.planparams.k = 0.01; 
acfg.visualize = 2;
acfg.viscfg.pause = .01;
acfg.datadir = pwd;
acfg.verb = 0;

acfg.reset = x0dubbins; acfg.datafile = 'classical_manonholon'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);

% %% Sequential communication: basic experiments: OP control
% 
% acfg = ACFG;
% acfg.idinfix = 'opseq'; 
% acfg.planparams.n = 200;
% acfg.comm = 'seq';
% 
% acfg.reset = x0{1}(:); acfg.datafile = 'magenmpc_seq_malin_1d_vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
% acfg.reset = x0{2}(:); acfg.datafile = 'magenmpc_seq_malin_1d_15vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
% acfg.reset = x0{3}(:); acfg.datafile = 'magenmpc_seq_malin_1d_25vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);


%% Sequential communication: influence of n // HP 8470p
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'budget';
% bcfg.legendprefix = 'Optimistic';
acfg = ACFG;
acfg.online = 1;
acfg.planparams.n = [25 50 75 100 200 300 400 500 600];
acfg.planparams.maxpredlength = Inf;

execbatchexp(bcfg.idinfix);


%% Sequential communication: influence of length of communicated predictions // F6Ve

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'predlen';
bcfg.in = 'planparamsmaxpredlength';

acfg = ACFG;
acfg.online = 1;
acfg.planparams.n = 300;
acfg.planparams.maxpredlength = [0 1 2 3 4 Inf];

execbatchexp(bcfg.idinfix);

end
