function S = ceexp_eval(cfg)
% For CE fuzzy QI, currently only supports same number of centers on each axis
% This function is OBSOLETE when used with cerbfps experiments; use OPTIMEXP instead

if nargin < 1, cfg = struct; end;
% defaults
CFG.synth = 1;              % perf stats
CFG.runtime = 0;            % run time
CFG.niter = 0;              % # of iterations
CFG.int = 'minmax';         % 'minmax'; or 'conf' for 95% confidence intervals

CFG.beststat = 0;           % plot stats of best run
CFG.worststat = 0;          % plot stats of worst run
CFG.besth = 0;              % plot stats of best run
CFG.worsth = 0;             % plot policy of worst run

CFG.problem = 'hillcar_problem';    % which problem
CFG.alg = 'cerbfqi';        % what algorithm (cerbfqi, cefuzzyqi, or cerbfps)
CFG.type = 'tvr';           % what type of algorithm for cerbfqi (full, tvr)
CFG.cost = 'mc';            % cost eval for cerbfqi
CFG.X0 = 'zero';            % what X0 to load
CFG.showX0 = [];            % what X0 to show (by default X0)
CFG.W = [];     
CFG.forcerecompute = 0;     % force score recomputation, even if X0 = showX0
CFG.forcensim = [];         % force number of simulations to perform
CFG.infix = '';             % infix of data files
CFG.postfix = '';           % postfix of data files

CFG.startat = 1;            % displacement of iteration index
CFG.N = [];                 % explicit range of N (#basis functions) if known
CFG.Np = [];                % same, for dimension-wise basis functions
CFG.Nmax = 20;              % up to how many basis functions to search data files
CFG.Jstar = [];             % optimal cost to compare to    
CFG.algopt = [];            % call the algorithm function with this string
CFG.plotopt = [];           % call the plot function of the problem with this string
CFG.runs = [];              % which runs to plot
CFG.runsmax = 10;           % how many runs to search for at most
CFG.runrestrict = [];       % process additional calls (plot etc.) only for these runs
CFG.Nrestrict = [];         % process additional calls only for these Ns
CFG.noconsole = -1;         % no console output
CFG.logy = 0;               % logarithmic y scale

% plot config
CFG.ylim = [];              % limits for axes
CFG.xlim = [];
CFG.ytick = [];             % explicit tick spacing
CFG.addtocrt = 0;           % add to current figure

% save configuration
CFG.grayscale = 0;
CFG.plottarget = 'screen';    % 'latex', 'beamer', 'screen', ''
CFG.savedir = '';
CFG.savefig = 'ceexpeval';    % filename for saving figure

cfg = checkparams(str2cfg(cfg, CFG), CFG); 
% defaults for N, Np
if isempty(cfg.N), cfg.N = 2:cfg.Nmax; end;
if isempty(cfg.Np), cfg.Np = 2:cfg.Nmax; end;
if isempty(cfg.runs),
    cfg.runs = 1:cfg.runsmax;
end;
if isempty(cfg.showX0), 
    cfg.showX0 = cfg.X0;
end;
cfg.grayscale = grayscalefromconfig(cfg);       % get grayscale
% noconsole default: true when plotting for app
if (strcmp(cfg.plottarget, 'latex') || strcmp(cfg.plottarget, 'beamer')) ...
    && cfg.noconsole == -1, cfg.noconsole = 1;
end;

cfg

switch cfg.alg,
    case {'cerbfps','cerbfqi'}
        range = cfg.N;
        nlabel = 'N';
    case 'cefuzzyqi',
        range = cfg.Np;
        nlabel = 'Np';
end;

% pre-establish algorithm function to call
if strcmp(cfg.alg, 'cerbfps'),  % use compatibility version of cerbfps
    algfun = 'cerbfps_old';
else                            % use the algorithm name
    algfun = cfg.alg;
end;

% determine if we need to recompute scores for a different X0
recompute = ~isequal(cfg.X0, cfg.showX0) || ~strcmp(cfg.cost, 'mc') || cfg.forcerecompute;

% get the explicit matrix of states from the problem if it was specified as a string
if recompute && ischar(cfg.showX0),
    cX0 = feval(cfg.problem, 'ce', cfg.showX0);
    cfg.showX0 =  cX0.X0;
end;
      
% stats structure
S = {};

% Main iterator
for n = range,
    % try looking for iteration-postfixed or no-iter postfix file
    if ~exist([ceconfig2filename(cfg, n, nlabel, 0) '.mat'], 'file') ...
            && ~exist([ceconfig2filename(cfg, n, nlabel, 1) '.mat'], 'file'), 
        disp(['No files for ' ceconfig2filename(cfg, n, nlabel, 1) ]);
        continue;   % no files for this n, try the next
    end;  
    J = []; K = []; filenames = {}; truns = [];
    for i = cfg.runs,        
        % form filename
        fname = ceconfig2filename(cfg, n, nlabel, i);
        % allow for "holes in the sequence", try next run index i
        if ~exist([fname  '.mat'], 'file'), continue; end;
        % load file including config
        cfg2 = cfg;
        load(fname, 'model', 'Jstar', 'k', 'trun', 'cfg', 'conv');
        expcfg = cfg; cfg = cfg2; clear cfg2; 
        if ~conv,
            disp(['Lack of convergence in ' fname]);
        end;
        % store optimal cost found and number of iterations taken
        J(end+1) = Jstar;
        K(end+1) = k-1; truns(end+1) = trun;
        filenames{end+1} = fname;
        % process stats (plots)
        if (isempty(cfg.runrestrict) || any(cfg.runrestrict == i)) ...
                && (isempty(cfg.Nrestrict) || any(cfg.Nrestrict == n)),
            if ~isempty(cfg.algopt),
                feval(algfun, [cfg.algopt ' datafile=' fname ' plottarget=screen verb=0']); 
            end;
            if ~isempty(cfg.plotopt), 
                feval(model2plotfun(model), [cfg.plotopt ' datafile=' fname ' plottarget=screen']); 
            end;
        end;
        % check if need to recompute score
        if recompute,
            mcfun = determineMCfun(cfg, expcfg, model);
            if ~isempty(cfg.forcensim), % force number of simulations
                expcfg.mc_nsim = cfg.forcensim;
            end;
            % obtain a noise realization if cfg.W is a random seed
            if ~isempty(cfg.W) && isscalar(cfg.W),
                rand('state', cfg.W);
                cfg.W = feval(model.genwfun, model, cfg.showX0, expcfg.mc_nsim, expcfg.mc_maxsteps);
            end;
            switch cfg.alg,
                case 'cefuzzyqi',
                    load(fname, 'cgridstar', 'thetastar', 'model');     
                    J(end) = feval(mcfun, cgridstar, thetastar, model, cfg.showX0, expcfg);
                case 'cerbfps',
                    load(fname, 'phistar', 'model');
                    cstar = phistar(1:model.p, :);
                    radstar = phistar(model.p+1:2*model.p, :);
                    uind = binvec2decx(phistar(2*model.p+1:end, :), 1);
                    if ~isempty(cfg.W),
                        % use same noise sequences for a fair comparison
                        J(end) = feval(mcfun, cstar, radstar, uind, model, cfg.showX0, expcfg, cfg.W);
                    else 
                        J(end) = feval(mcfun, cstar, radstar, uind, model, cfg.showX0, expcfg);
                    end;
            end;
        end;
    end;
    stat.(nlabel) = n; stat.J = J; stat.K = K; stat.fnames = filenames; stat.trun = truns;
    S{end+1} = stat;
end;

% process, compute statistics
Jevol = zeros(3, length(S)); Jstd = zeros(1, length(S));
Kevol = 0*Jevol; Kstd = 0*Jstd; 
nspan = 0*Jstd; nexp  = 0*Jstd;
Tevol = 0*Jevol;
minp.J = inf; maxp.J = -inf;
for i = 1:length(S),
    % number of experiments ran for given value of n
    nexp(i) = length(S{i}.J);
    nspan(i) = S{i}.(nlabel);
    % cost statistics
    Jevol(:, i) = [min(S{i}.J) mean(S{i}.J) max(S{i}.J)];
    Jstd(i) = std(S{i}.J, 1);
    % # of iter statistics
    Kevol(:, i) = [min(S{i}.K) mean(S{i}.K) max(S{i}.K)];
    Kstd(i) = std(S{i}.K, 1);
    % # run time statistics
    Tevol(:, i) = [min(S{i}.trun) mean(S{i}.trun) max(S{i}.trun)];
    if Jevol(1, i) < minp.J,
        minp.J = Jevol(1, i);
        minp.fname = S{i}.fnames{find(S{i}.J == minp.J, 1)};
    end;
    if Jevol(3, i) > maxp.J,
        maxp.J = Jevol(3, i);
        maxp.fname = S{i}.fnames{find(S{i}.J == maxp.J, 1)};
    end;
end;

figh = [];

% ---------------------
% Process plots

% grayscale styles
if cfg.addtocrt, 
    gs.avgstyle = {['-'], 'Color', 'k', 'LineWidth', 1};
    gs.lowerboundstyle = {['--'], 'Color', [.3 .3 .3], 'LineWidth', 1};
    gs.upperboundstyle = {['-.'], 'Color', [.3 .3 .3], 'LineWidth', 1};
    gs.optstyle = {'-', 'Color', 'k', 'LineWidth', 2};
else
    gs.avgstyle = {['-o'], 'Color', 'k', 'MarkerEdgeColor', 'k', 'LineWidth', 1.5};
    gs.lowerboundstyle = {['--o'], 'Color', [.5 .5 .5], 'MarkerSize', 5, 'MarkerEdgeColor', [.3 .3 .3], 'LineWidth', 1.5};
    gs.upperboundstyle = {['-.o'], 'Color', [.5 .5 .5], 'MarkerSize', 5, 'MarkerEdgeColor', [.3 .3 .3], 'LineWidth', 1.5};
    gs.optstyle = {'-', 'Color', 'k', 'LineWidth', 2};
end;
% black to dirty white
gs.cm = gray(96); gs.cm = gs.cm(24:end);
if cfg.addtocrt,
    cs.avgstyle = {['-'], 'Color', 'b', 'LineWidth', 1.5};
    cs.lowerboundstyle = {['-'], 'Color', 'r', 'LineWidth', 1.5};
    cs.upperboundstyle = {['-'], 'Color', 'g', 'LineWidth', 1.5};
    cs.optstyle = {'-', 'Color', 'k', 'LineWidth', 2};
else
    cs.avgstyle = {['-o'], 'Color', 'k', 'MarkerEdgeColor', 'b', 'LineWidth', 1.5};
    cs.lowerboundstyle = {['--o'], 'Color', .7+[0 0 .3], 'MarkerEdgeColor', 'r', 'LineWidth', 1.5};
    cs.upperboundstyle = {['-.o'], 'Color', .7+[0 0 .3], 'MarkerEdgeColor', 'g', 'LineWidth', 1.5};
    cs.optstyle = {'-', 'Color', 'k', 'LineWidth', 2};
end;
cs.cm = jet(64);
% set style
if cfg.grayscale, sty = gs; 
else sty = cs; end;

% readable labels
commonprop = {'Interpreter', 'LaTeX', 'FontSize',13};
rl.single = 'score'; 
rl.avg = 'mean score';  rl.max = 'max score';  rl.min = 'min score';  rl.opt = 'optimal score';
rl.tsingle = 'execution time';
rl.tavg = 'mean execution time'; rl.tmax = 'max execution time';  rl.tmin = 'min execution time';
rl.isingle = '# iterations';
rl.iavg = 'mean # iterations'; rl.imax = 'max # iterations';  rl.imin = 'min # iterations';

names = fieldnames(rl);
for i = 1:length(names),
    rl.([names{i} 'x']) = ['CE ' rl.(names{i})];
end;

switch cfg.alg,
    case {'cerbfps','cerbfqi'}
        rl.x = {'Number of RBFs \it{N}'};
    case 'cefuzzyqi',
        rl.x = {'Number of fuzzy sets on each axis'};
end;
rl.y = {'Score'};
rl.ty = {'Execution time [s]'};
rl.itery = {'Iterations'};
labels = rl;    % no psfrag labels


% --------------------
% synthetic stats
if cfg.synth,
    if cfg.addtocrt,
        figh(end+1) = gcf; hold on;
    else 
        figh(end+1) = figure;
    end;
    nolegend = 0;
    if any(nexp > 1), 
        plot(nspan, Jevol(2, :), sty.avgstyle{:}); 
        hold on;
        if cfg.int(1) == 'm',       % min max interval
            plot(nspan, Jevol(3, :), sty.upperboundstyle{:}); plot(nspan, Jevol(1, :), sty.lowerboundstyle{:});
        elseif cfg.int(1) == 'c',   % 95% confidence interval
            % NOTE THIS WAY OF COMPUTING THE CONF INT IS INCORRECT< NEED TO DIVIDE BY SQRT(N)
            plot(nspan, Jevol(2, :)+1.96*Jstd, sty.upperboundstyle{:}); 
            plot(nspan, Jevol(2, :)-1.96*Jstd, sty.lowerboundstyle{:});
        end;
        if ~cfg.addtocrt,
            leg = {labels.avg, labels.max, labels.min};
        else 
            leg = {labels.avgx, labels.maxx, labels.minx};
        end
    else
        nolegend = 1;
        plot(nspan, Jevol(2, :), sty.avgstyle{:}); hold on;
        if ~cfg.addtocrt,
            leg = {labels.single};
        else 
            leg = {labels.singlex};
        end
    end;
    if ~isempty(cfg.Jstar),
        plot([nspan(1) nspan(end)], [cfg.Jstar cfg.Jstar], sty.optstyle{:});
        leg{end+1} = labels.opt;
    end;

    setlimits(cfg);
    t = [cfg.problem ' ' cfg.alg ' ' cfg.type ' ' cfg.X0 ' ' cfg.postfix];
    if ~cfg.addtocrt,
        if ~nolegend, legend(leg, 'Location', 'Best'); end;
        xlabel(labels.x{:}); ylabel(labels.y{:}); 
        set(gca, 'XTick', nspan);
    else
        lh = legend;
        text = get(lh, 'String');
        legend(text{:}, leg{:}, 'Location', 'Best');
    end;
    if strcmp(cfg.plottarget, 'screen'), title(t); end;
    set(figh, 'name', t, 'numberTitle', 'off');

end;

% --------------------
% run time
if cfg.runtime,
    if cfg.addtocrt,
        figh(end+1) = gcf; hold on;
    else 
        figh(end+1) = figure;
    end;
    setlimits(cfg);
    nolegend = 0;
    if cfg.logy, plotfun = @semilogy; 
    else plotfun = @plot;
    end;
    if any(nexp > 1),
        feval(plotfun, nspan, Tevol(2, :), sty.avgstyle{:}); 
        hold on;
        plotfun(nspan, Tevol(3, :), sty.upperboundstyle{:}); plotfun(nspan, Tevol(1, :), sty.lowerboundstyle{:});
        if ~cfg.addtocrt,
            leg = {labels.tavg, labels.tmax, labels.tmin};
        else
            leg = {labels.tavgx, labels.tmaxx, labels.tminx};
        end;
    else
        nolegend = 1;
        plotfun(nspan, Tevol(2, :), sty.avgstyle{:}); hold on;
        if ~cfg.addtocrt,
            leg = {labels.tsingle};
        else
            leg = {labels.tsinglex};
        end
    end;

    setlimits(cfg);
    t = [cfg.problem ' ' cfg.alg ' ' cfg.type ' ' cfg.X0 ' ' cfg.postfix];
    if ~cfg.addtocrt,
        if ~nolegend, legend(leg, 'Location', 'Best'); end;
        xlabel(labels.x{:}); ylabel(labels.ty{:}); 
        set(gca, 'XTick', nspan);
    else
        lh = legend;
        text = get(lh, 'String');
        legend(text{:}, leg{:}, 'Location', 'Best');
    end;
    if strcmp(cfg.plottarget, 'screen'), title(t); end;
    set(figh, 'name', t, 'numberTitle', 'off');
end;

% --------------------
% #iter
if cfg.niter,
    if cfg.addtocrt,
        figh(end+1) = gcf; hold on;
    else 
        figh(end+1) = figure;
    end;
    setlimits(cfg);
    nolegend = 0;
    plotfun = @plot;
    if any(nexp > 1),
        feval(plotfun, nspan, Kevol(2, :), sty.avgstyle{:}); 
        hold on;
        plot(nspan, Kevol(3, :), sty.upperboundstyle{:}); plot(nspan, Kevol(1, :), sty.lowerboundstyle{:});
        if ~cfg.addtocrt,
            leg = {labels.iavg, labels.imax, labels.imin};
        else
            leg = {labels.iavgx, labels.imaxx, labels.iminx};
        end;
    else
        nolegend = 1;
        feval(plotfun, nspan, Kevol(2, :), sty.avgstyle{:}); hold on;
        if ~cfg.addtocrt,
            leg = {labels.isingle};
        else
            leg = {labels.isinglex};
        end
    end;

    setlimits(cfg);
    t = [cfg.problem ' ' cfg.alg ' ' cfg.type ' ' cfg.X0 ' ' cfg.postfix];
    if ~cfg.addtocrt,
        if ~nolegend, legend(leg, 'Location', 'Best'); end;
        xlabel(labels.x{:}); ylabel(labels.itery{:}); 
        set(gca, 'XTick', nspan);
    else
        lh = legend;
        text = get(lh, 'String');
        legend(text{:}, leg{:}, 'Location', 'Best');
    end;
    if strcmp(cfg.plottarget, 'screen'), title(t); end;
    set(figh, 'name', t, 'numberTitle', 'off');
end;

% -------------------------
% Best or worst convergence or policy
if cfg.besth || cfg.worsth || cfg.beststat || cfg.worststat,    
    pcfg.verb = 0;
    pcfg = copyfields(cfg, pcfg, {'plottarget', 'savedir', 'savefig'});
  
    if cfg.beststat,
        pcfg.datafile = maxp.fname; 
        pcfg.stats = 1;
        feval(algfun, pcfg);
    end;
    if cfg.besth,
        pcfg.datafile = maxp.fname; 
        pcfg.fuzzyh = 1;
        pcfg.basis = 1;
        feval(@hillcar_plot, pcfg);   % hardcoded!!
    end;
    
    if cfg.worststat,
        pcfg.datafile = minp.fname; 
        pcfg.stats = 1;
        feval(algfun, pcfg);
    end;
    if cfg.worsth,
        pcfg.datafile = minp.fname; 
        pcfg.fuzzyh = 1;
        pcfg.basis = 1;
        feval(@hillcar_plot, pcfg);     % hardcoded!!
    end;
    % already saved files via function call
    cfg.plottarget = '';
end;

% console output
if ~cfg.noconsole,
    % for which ns were experiments ran
    nspan
    % number of ran experiments
    nexp
    % the cost mean
    Jevol
    % and the mean of the number of runations to convergence
    Kevol
    %
    Tevol
end;

% save last figure if indicated
if ~isempty(figh), 
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
end;


end % ceexp_eval() ------------------------------------------------

% Local functions

function mcfun = determineMCfun(cfg, expcfg, model)

    switch cfg.alg,
        case 'cerbfps',
            switch expcfg.actsel,
                case 'max',
                    if isfield(model, 'mc_rbfnearestpolicy_fun'),
                        mcfun = model.mc_rbfnearestpolicy_fun;
                    else 
                        mcfun = @mc_rbfnearestpolicy;
                    end;
                case 'voting',
                    if isfield(model, 'mc_rbfnearestpolicy_fun'),
                        mcfun = model.mc_rbfnearestpolicy_fun;
                    else 
                        mcfun = @mc_rbfnearestpolicy;
                    end;
            end;

        case 'cerbfqi',
            if isfield(model, 'mc_rbfq_fun'),
                mcfun = model.mc_rbfq_fun;
            else 
                mcfun = @mc_rbfq_fun;
            end;

        case 'cefuzzyqi',
            if isfield(model, 'mc_fuzzyqiter_fun'),
                mcfun = model.mc_fuzzyqiter_fun;
            else 
                mcfun = @mc_fuzzyqiter;
            end;
    end;

end     % determineMCfun()