function varargout = lspibatch(cfg)
% Semi-online, batch least-squares policy iteration.
%   VARARGOUT = LSPI(CFG)
% A semi-online, batch implementation of least-squares policy iteration [1].
% OBSOLETE: This function should no longer be used, the idea didn't work out.
%
% Inputs:
%   CFG             - structure with fields as commented in the code below
%           can also be given as a string, see str2cfg
%
% [1] Lagoudakis, M. G. & Parr, R. "Least-Squares Policy Iteration"
%   Journal of Machine Learning Research, 2003, 4, 1107-1149

% Author: Lucian Busoniu
% Version: ---


if nargin < 1, cfg = struct(); end;

% ==== DECLARE CONFIGURATION DEFAULTS ====
% function config
CFG.run = 0;                        % run learning
CFG.resume = 0;                     % resume learning
CFG.replay = 0;                     % replay learned policy
CFG.sol = 0;                        % plot solution and convergence stats
CFG.evol = 0;                       % plot evolution statistics
CFG.datafile = 'lspidata';          % save data to file
CFG.datadir = [];                   % data dir
% main algorithm config
CFG.problem = '';                   % what problem to solve
CFG.approx = [];                    % approximator object (or config for approx)
CFG.gamma = [];                     % discount factor
CFG.eps = 0.01;                     % convergence threshold
CFG.maxiter = 100;                  % maximum # of iterations
CFG.method = 'inv';                 % parameter computation method: currently only 'inv' supported
CFG.invdelta = 0;                   % use for inverse computation
CFG.h0 = [];                        % start with this initial policy
CFG.h0args = {};                    % arguments for the initial policy
% samples config
CFG.samples = [];                   % initial samples object (or config for generate_samples)
CFG.loadsamples = [];               % load initial samples from this file
CFG.diter = 1;                      % perform this # of iterations in-between sampling sessions
CFG.dNs = [];                       % add this # of samples in each session
CFG.Nsmax = [];                     % target # of samples (once Ns >= Nsmax, stop sampling)
CFG.dsamples = struct;              % configuration for sampling sessions (method, exploration, etc)
% replay config
CFG.x0 = [];                        % initial state for replay (otherwise the problem default or zeros)
CFG.tend = 30;                      % end time for replay
% stats config
CFG.storeab = 0;                    % whether to store A and b histories
% display config
CFG.verb = 3;                       % verbosity: the higher, the more detailed the messages displayed
CFG.iterdisp = 1;                   % feedback after every iterdisp iterations
CFG.itersave = 3;                   % save after each itersave iterations
CFG.sampledisp = .05;               % display progress after this % of samples processed
CFG.silent = 0;                     % suppress all output
% figure config
CFG = setfigprop(CFG, 'addfields');
CFG.miscinfo = '';                  % field for miscellaneous information

% Early defaults (initialized before calling problem defaults)
ECFG.model_params = {};             % parameters for problem calling in 'model' mode
ECFG.lspi_params = {};              % parameters for problem calling in 'lspi' mode

% List of fields that define the problem
KEEPFIELDS = {'problem', 'gamma', 'approx', 'samples', 'usesparse'};

% ==== PARSE AND PROCESS CONFIG ====
cfg = parseconfig(cfg, CFG, ECFG, 'lspi');
% get environment (Matlab, hardware) info
cfg.envinfo = getenvx;

% If running, check presence of data file, warn on possible overwrite
if cfg.run && exist([cfg.datafile '.mat'], 'file'),
    reply = input(['File [' cfg.datafile '.mat] already exists (possible overwrite). Continue? Y/N [N]: '], 's');
    if isempty(reply) || reply == 'N', return; end;
end;       
% determine if initialization needed; if not, need to load: check if datafile exists
cfg.init = ~(cfg.resume || cfg.replay || cfg.sol || cfg.evol);
if ~cfg.init && ~exist([cfg.datafile '.mat'], 'file'),
    error(['File [' cfg.datafile '.mat] does not exist. Terminating.']);
end;

% ==== IF NOT INITIALIZING: NEED TO LOAD DATA ====
if ~cfg.init,        % load data file, making sure that cfg and KEEPFIELDS is not overwritten
    cfg1 = cfg; kf = KEEPFIELDS;
    load(cfg.datafile);
    % Overwrite problem-defining fields from loaded config, keep the rest as in the (current) cfg1;
    % the result becomes the new config
    cfg = copyfields(cfg, cfg1, kf);
    KEEPFIELDS = kf; clear cfg1 kf;
    dispx(['Data loaded from [' cfg.datafile '].'], cfg.verb, 1);
    if exist('htheta', 'var'), 
        error('The algorithm used to produce this data was not LSPI, but LSPIH. Please use LSPIH.');
    end;
end;

if ~strcmp(cfg.method, 'inv'), error('Currently only normal inversion (INV) method supported.'); end;

% Echo config
dispx('LSPI will run with the following configuration:', cfg.verb, 1);
dispx(cfg, cfg.verb, 1, 'cfg', 'config');


% ==== IF INITIALIZING: CREATE MODEL, AND IF NEEDED: APPROXIMATOR, SAMPLES ====
if cfg.init,
    model = feval(cfg.problem, 'model', cfg.model_params{:});
    % heuristic check if we already have an approx object
    if isstruct(cfg.approx) && isfield(cfg.approx, 'N') && isfield(cfg.approx, 'phi') ...
            && isfield(cfg.approx, 'q') && isfield(cfg.approx, 'h'),
            % nothing to do
            dispx('Approx object supplied', cfg.verb, 3);
    else    % create approximator using cfg.approx as the config
        cfg.approx = create_approx(model, cfg.approx);
    end;
    % samples object
    if ~isempty(cfg.loadsamples) && exist([cfg.loadsamples '.mat'], 'file'),
        % try loading samples -- performs no checking whether the same type & number of
        % samples is required
        loadwithprefix(cfg.loadsamples, 's_', 'cfg');
        cfg.samples = s_cfg.samples;
        clear s_cfg;
        dispx(['Initial samples object loaded from [' cfg.loadsamples ']'], cfg.verb, 1);
    else
        % heuristic check if we already have a samples collection supplied
        if isstruct(cfg.samples) && isfield(cfg.samples, 'N') && isfield(cfg.samples, 'X') ....
                && isfield(cfg.samples, 'U') && isfield(cfg.samples, 'Xp') ...
                && isfield(cfg.samples, 'R'),
                dispx('Initial samples object supplied', cfg.verb, 3);
                % nothing to do
        else    % generate samples using cfg.samples as the config
            dispx('Generating initial samples...', cfg.verb, 0);
            cfg.samples = generate_samples(model, cfg.samples, cfg.approx);
            dispx([8 ' done.'], cfg.verb, 0);
        end;
    end;
    % initial policy
    if ~isempty(cfg.h0),
        % determine the order of the possibly dynamic controller
        try     [cx0, cord] = feval(cfg.h0, 'init', cfg.h0args{:});
        catch   cord = 0;   % compatibility for old-style static policies
        end;
        if cord > 0, error('Cannot control arbitrary samples with a dynamical controller'); end;    
    end;
end;

% cfg.approx
% cfg.samples

% ==== RUN LSPI ====
if cfg.run || cfg.resume,
       
    % shorthand variables
    approx = cfg.approx;
    N = approx.N;
    Ns = cfg.samples.N; dNs = cfg.dNs;
    Xs = cfg.samples.X; Us = cfg.samples.U; Xps = cfg.samples.Xp; Rs = cfg.samples.R;

    % Initialize if not resuming
    if cfg.init,
        % History
        if cfg.storeab,
            Ah = cell(cfg.maxiter, 1);
            bh = cell(cfg.maxiter, 1);
        end;
        thetah = cell(cfg.maxiter+1, 1);
        deltah = nan(cfg.maxiter+1, 1);      % deltah(1) will always remain NaN

        % if worst-case storage space for pre-computed BF values of all the (x, u) samples
        % is reasonable, pre-init; for now, pre-init is hardcoded
        storephi = 1; %(N * Ns * 8) <= 300e6;   % 300 MB with double (8 bytes) storage
        % use sparse A and PHI if the approx type recommends it
        if isfield(approx, 'sparse') && approx.sparse, 
            cfg.usesparse = 1; zeromatrix = @sparse;
        else
            cfg.usesparse = 0; zeromatrix = @zeros;
        end;
        if storephi,
            dispx('Pre-computing BF values for the initial samples...', cfg.verb, 0);
            PHI = zeromatrix(N, Ns);
            for i = 1:Ns, PHI(:, i) = approx.phi(approx, Xs(:, i), Us(:, i)); end;
            dispx([8 ' done.'], cfg.verb, 0);
        else
            dispx('WARNING! Cannot store BF values. Algorithm will run slowly.', cfg.verb, 0);
        end;
        
        k = 1;                  % iteration index
        dk = 1;                 % iteration index after last sampling session
        dsamplesk = [];         % remembers before which iterations were samples generated
        trun = 0;               % execution time
        condA = nan(cfg.maxiter, 1);  % condition number history for A
        theta = zeros(N, 1);    % initialize parameter vector to zeroes
    end;        % initialization IF

    dispx('Performing LSPI...', cfg.verb, 0);
    
    % check if should use an initial policy
    useh0 = ~isempty(cfg.h0);
    if useh0;
        disc = any(strcmp(approx.type, {'rbfdisc', 'triang'}));
        if disc,        % initialize action grids and their size
            U = approx.U; 
            Un = zeros(length(U)); for i=1:length(U), Un(i) = length(U{i}); end;        
        end;
    end;
    % Main policy iteration loop
    conv = 0; Anoninv = 0; tmark = cputime;
    while ~conv && ~Anoninv && k <= cfg.maxiter,
        % check if we need to run a sampling session
        if (dk > cfg.diter) && (Ns < cfg.Nsmax),
            % these appends be made more efficient by preinitializing the memories; but 
            % algorithm may converge before we fill in Nsmax samples & then we're wasting space
            %
            % configure and generate new samples
            dsampcfg.N = dNs;
            dsampcfg.policy = @lspi_policy;
            dsampcfg.policyargs = {approx, theta};
            dsampcfg = copyfields(cfg.dsamples, dsampcfg); % other arguments
            dispx(['Generating ' num2str(dNs) ' new samples...'], cfg.verb, 1);
            dsamp = generate_samples(model, dsampcfg, approx);
            dispx([8 ' done.'], cfg.verb, 1);
            % append the new samples
            Ns = Ns + dNs;
            Xs = [Xs dsamp.X]; Us = [Us dsamp.U]; Xps = [Xps dsamp.Xp]; Rs = [Rs dsamp.R];
            clear dsamp;
            % compute the basis function values for the new samples
            if storephi,
                dispx('Pre-computing BF values for the new samples...', cfg.verb, 1);
                % the first assignment in the FOR loop also enlarges PHI
                for i = Ns:-1:(Ns-dNs+1), PHI(:, i) = approx.phi(approx, Xs(:, i), Us(:, i)); end;
                dispx([8 ' done.'], cfg.verb, 1);
            else dispx('WARNING! Cannot store BF values for the new samples.', cfg.verb, 1);
            end;
            % remember iteration where we generated; and reset the in-between iteration counter
            dsamplesk(end+1) = k;
            dk = 1;
        end;
        
        % save theta on the parameter history 
        thetah{k} = theta;
        
        % init A and b
        if cfg.invdelta > 0,
            if cfg.usesparse,   A = sparse(1:N, 1:N, cfg.invdelta, N, N);
            else                A = eye(N, N) .* cfg.invdelta;
            end;
        else
            A = zeromatrix(N, N);
        end;
        b = zeros(N, 1);        % b needs not be sparse
        
        % Loop through samples, updating A and b
        progs = cfg.sampledisp; dispx(sprintf('LSTD-Q k=%d iter, samples processed: %2d%%', k, 0), cfg.verb, 3);
        for i = 1:Ns,
            % compute phi(x, u)
            phixu = PHI(:, i);  % removed test below, should be added when storephi may be false
%             if storephi,        phixu = PHI(:, i);
%             else                phixu = approx.phi(approx, Xs(:, i), Us(:, i));
%             end;
            % find out the action in x'
            if k == 1 && useh0,         % use an initial policy
                up = cfg.h0(Xps(:, i), [], cfg.h0args{:});  % [] = undef time arg
                if disc, up = quantize(up, U, Un); end;     % quantize action if discrete-action approx
            else                        % use the current poolicy
                up = approx.h(approx, theta, Xps(:, i));
            end;
            % compute phi(x', u')
            phixpup = approx.phi(approx, Xps(:, i), up);
            % update A and b
            A = A + phixu * (phixu - cfg.gamma * phixpup)';
            b = b + phixu * Rs(i);
            if i/Ns >= progs,    % display progress inline
                dispx([8 8 8 8 sprintf('%2d%%', fix(i/Ns*100))], cfg.verb, 3);
                progs = progs + cfg.sampledisp;
            end;
        end;
        dispx([8 8 8 8 '100%. Solving LSQR...'], cfg.verb, 3);
        lastwarn('');
        theta = A \ b;
        if ~isempty(lastwarn), Anoninv = 1; end;    % set non-invertibility flag 
        dispx([8 ' done.'], cfg.verb, 3);
        deltah(k+1) = norm(theta - thetah{k}, 2);
        
        % check convergence condition
        conv = deltah(k+1) <= cfg.eps;
        % update runtime (the statistics and user feedback commands below are not counted in
        % the stats)
        trun = trun + (cputime - tmark);
        
        % compute and store condition number for A
        if cfg.usesparse, condA(k) = condest(A); else condA(k) = cond(A); end;
        % store A and b if configured to do so
        if cfg.storeab, Ah{k} = A; bh{k} = b; end;
        
        % visual feedback of algorithm progress
        if ~mod(k, cfg.iterdisp),
            dispx(['k=' num2str(k) ' LSPI iter done, cond(A)=' num2str(condA(k))...
                ' delta_k+1=' num2str(deltah(k+1))], cfg.verb, 2);
        end;
        % data backup
        if ~mod(k, cfg.itersave),            
            save(cfg.datafile);
            dispx(['Data at iter k=' num2str(k) ' saved to [' cfg.datafile '].'], cfg.verb, 1);
        end;
        
        tmark = cputime;
        k = k + 1; dk = dk + 1;
    end;        % WHILE not converged and allowed more iterations

    if conv,        dispx('Convergence detected. Algorithm stopped.', cfg.verb, 0);
    elseif Anoninv, dispx('The A matrix is singular or badly conditioned. Algorithm stopped.', cfg.verb, 0);
    else            dispx(['maxiter=' num2str(cfg.maxiter) ' exhausted. Algorithm stopped'], cfg.verb, 0);
    end;
    
    % add last theta to the history
    thetah{k} = theta;
end;


% Options below might create figures, init figure handles array
figh = [];

% ==== REPLAY POLICY ====
if cfg.replay,

    % initial state
    if ~isempty(cfg.x0), 
        if ischar(cfg.x0) && ~isempty(cfg.problem),
            % use the model function to get a string-named initial state
            x0 = feval(cfg.problem, 'x0', cfg.x0);
        else
            x0 = cfg.x0(:);         % specified initial state
        end;
    else    % try getting default initial state
        try         x0 = feval(cfg.problem, 'x0');
        catch       x0 = zeros(model.p, 1);    % zeros
        end;
    end;         
    dispx(['Controlling from x0=' num2str(reshape(x0, 1, [])) ], cfg.verb, 0);

    % init history
    t = 0 : model.Ts : cfg.tend;
    Ns = length(t)-1;       % number of samples / time instances at which control is applied
    x = zeros(model.p, length(t)); x(:, 1) = x0;
    u = zeros(model.q, length(t)); u(:, end) = NaN;
    r = zeros(1, length(t)); r(1) = NaN;

    % steps loop
    for k = 1:Ns,
        % compute action
        u(:, k) = approx.h(approx, theta, x(:, k));
        % apply to system
        [x(:, k+1) r(k+1) terminal] = feval(model.fun, model, x(:, k), u(:, k));
        if terminal, Ns = k; break; end;      % entered terminal state
    end;
 
    % plot history & optionally save figures
    hist.t = t(1:Ns+1); hist.x = x(:, 1:Ns+1); hist.u = u(:, 1:Ns+1); hist.r = r(1:Ns+1);
    hist.R = discreturn(cfg, hist.r, Ns, terminal);
    if ~cfg.silent,
        if isfield(model, 'plotfun'),
            mpfigh = feval(model.plotfun, hist);
            if ~isempty(mpfigh), figh(end+1) = mpfigh; end;
        else
            figh(end+1) = plothistory(hist);
        end;
%         % also put the return in the figure title
%         title(['R(x_0)=' num2str(hist.R(1))]);
        setfigprop(cfg);
        % save if requested
        saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
    end;
end;        % IF replay

% ==== STATISTICS ====
if (cfg.evol || cfg.sol) && ~cfg.silent,
    % grayscale styles
    gs.cm = gray(96); gs.cm = gs.cm(24:end);
    % color styles
    cs = gs;
    cs.cm = jet(64);
    % set style
    if cfg.grayscale, sty = gs; else sty = cs; end;    
    
    % readable labels
    labels.iter = 'Iteration'; 
    labels.h = 'h(x)'; labels.V = 'V(x)'; 
    labels.delta = {'$\Vert \theta_\ell - \theta_{\ell-1}\Vert_2$', 'Interpreter', 'Latex', 'FontSize', 14}; 
    labels.condA = 'cond(A)';
end;

if cfg.sol && ~cfg.silent,
    if exist('Anoninv', 'var') && Anoninv,
        dispx('A was non-invertible, solution is meaningless. Skipping plot.', cfg.verb, 0);
    else
        figh(end+1) = figurex([1100 450]); % colormap(sty.cm);
        K = find(~isnan(deltah), 1, 'last') - 1;
        subplot(3, 2, [1 3]); cla;
        approx.plotv(approx, theta); title(labels.V);
        subplot(3, 2, [2 4 6]); cla;
        approx.ploth(approx, theta, 'npoints=100'); title(labels.h);
        subplot(3, 2, 5); cla;
        semilogy(1:K, deltah(2:K+1), 'k'); grid on; xlabel(labels.iter); ylabel(labels.delta{:});
        setfigprop(cfg);
        % save if requested
        saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
    end;
end;
    
if cfg.evol && ~cfg.silent,
    if exist('cleanedup', 'var') && cleanedup >= 2,
        warning('Cannot replay evolution, hard cleanup was performed on the datafile.');
    elseif exist('Anoninv', 'var') && Anoninv,
        dispx('A was non-invertible. Skipping evolution plot.', cfg.verb, 0);
    else
        figh(end+1) = figure; colormap(sty.cm);
        K = find(~isnan(deltah), 1, 'last') - 1;
        for k = 1:K+1,
            figcfg.figname = sprintf('Iter#%d, %s', k, cfg.datafile);
            setfigprop(figcfg);
            subplot(221); cla;
            approx.plotv(approx, thetah{k}, 'npoints=30'); title(labels.V);
            subplot(222); cla;
            approx.ploth(approx, thetah{k}); title(labels.h);
            subplot(223); cla;
            semilogy(1:k-1, deltah(2:k), 'k'); xlabel(labels.iter); ylabel(labels.delta{:});
            if k <= K && exist('condA', 'var'),   % A not defined after the final iteration
                subplot(224); cla;
                semilogy(1:k, condA(1:k), 'k'); xlabel(labels.iter); ylabel(labels.condA);
            end;
            if k <= K, pause; end;
        end;
        % save if requested
        saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
    end;
end;


% ==== BACKUP DATA ====
if cfg.run || cfg.resume, 
    % save into the same directory as the problem
    if isempty(cfg.datadir),
        datadir = fileparts(which(cfg.problem));
    else
        datadir = cfg.datadir; 
        if (datadir(end) == '\'), datadir = datadir(1:end-1); end;
    end;
    % if default file name is used, generate a unique file name
    if strcmp(cfg.datafile, CFG.datafile),
        c = fix(clock);
        for i = 1:length(c),
            cfg.datafile = [cfg.datafile '-' num2str(c(i), '%02d')];
        end;
        if exist([CFG.datafile '.mat'], 'file'), delete([CFG.datafile '.mat']); end;
    elseif ~strcmp(datadir, pwd)
        delete([cfg.datafile '.mat']);   % need to save in a different directory, so delete anyway
    end;    % otherwise just overwrite
    cfg.datafile = [datadir '\' cfg.datafile];
    save(cfg.datafile);
    dispx(['Data was saved to [' cfg.datafile '].'], cfg.verb, 1);
end;

% -----------------------------------------------
% set output
if cfg.run || cfg.resume,       % output optimal solution
    varargout = {theta};
elseif cfg.replay,              % output history and possibly figure handles
    varargout = {hist, figh};
elseif cfg.evol || cfg.sol,               % output fig handles (replay takes precedence)
    varargout = {figh};
end;

end
% lspi() RETURNING varargout =================================================================

function [u, ord] = lspi_policy(x, t, approx, theta)
% Local function for applying an LSPI policy
% process 'init' mode
if ischar(x) && strcmp(x, 'init'),
    u = []; ord = 0;   % the order of the controller
    return;
end;
u = approx.h(approx, theta, x);
end
