function figh = dc_fzvsrbf(cfg)
% Fuzzy versus RBF Q-iteration study for the DC motor
% Uses the most refined state & action gridding from the consistency experiments
% Written for the Fuzzy Q-iter jnl paper draft

CFG.N = 41;
CFG.M = 15;
% create the following files if needed
CFG.fzfile = 'fzdc_lqr_lin'; 
CFG.rbffile = 'rbfdc_lqr_lin'; 
CFG.savedir = 'D:\Work\tex\papers\fzqi\img\';
CFG.datadir = 'd:\work\m\approxrl\systems\dcmotor\dc_conv_fzvsrbf_lqr_lin\';
CFG.plottarget = 'screen';
CFG.qieps = 1e-5;
CFG.mceps = 0.1;
CFG.eval = 0;
CFG.X0 = 'coarse';
CFG.model_params = {'lqr', 'lin'};
CFG.hcolorbar = 1;
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG

cfg = parseconfig(cfg, CFG);

crtdir = pwd;
cd(cfg.datadir);

% common config
ccfg = struct;
ccfg.problem = 'dc_problem';
ccfg.xgrids = {-pi:2*pi/(cfg.N-1):pi, -16*pi:32*pi/(cfg.N-1):16*pi};
ccfg.ugrids = {-10:20/(cfg.M-1):10};
ccfg.gamma = 0.95;
ccfg.model_params = cfg.model_params;
ccfg.savetheta = 1;
ccfg.eps = cfg.qieps;
ccfg.plottarget = cfg.plottarget;
ccfg.savedir = cfg.savedir;

% fuzzy config
if ~exist([cfg.fzfile '.mat'], 'file')
    rcfg = ccfg;
    rcfg.serial = 0;
    rcfg.run = 1;
    rcfg.datafile = cfg.fzfile;

    fuzzyqi(rcfg);
end;

% rbf config
if ~exist([cfg.rbffile '.mat'], 'file')
    rcfg = ccfg;
    rcfg.run = 1;
    [rcfg.c, rcfg.rad] = gen_tvr_nrbfs(rcfg.xgrids, rcfg.gamma);
    rcfg.datafile = cfg.rbffile;

    rbfqi(rcfg);
end;


% Plot code
ccfg = cfg;     % for the plot fields
ccfg.savedir = cfg.savedir;
ccfg.gridres = 60;
ccfg.plottarget = cfg.plottarget;

figh = [];

% Fuzzy Q and h
pcfg = ccfg;
pcfg.datafile = cfg.fzfile;
pcfg.fuzzyq = 1;
pcfg.savefig = 'dcfz_q';
figh = [figh dc_plot(pcfg)];

pcfg = ccfg;
pcfg.datafile = cfg.fzfile;
pcfg.fuzzyh = 1;
pcfg.savefig = 'dcfz_h';
if cfg.hcolorbar, pcfg.colorbar = 1; end;
figh = [figh dc_plot(pcfg)];

pcfg = ccfg;
pcfg.traj = fuzzyqi(['replay datafile=' cfg.fzfile ' silent x0=[-pi,0] tend=1']);
pcfg.savefig = 'dcfz_traj';
pcfg.xlim = []; pcfg.ylim = []; pcfg.figsize = [500 550];
pcfg.box = 0;
figh = [figh dc_plot(pcfg)];

% RBF Q and h
pcfg = ccfg;
pcfg.datafile = cfg.rbffile;
pcfg.rbfq = 1;
pcfg.savefig = 'dcrbf_q';
figh = [figh dc_plot(pcfg)];

pcfg = ccfg;
pcfg.datafile = cfg.rbffile;
pcfg.rbfh = 1;
pcfg.savefig = 'dcrbf_h';
if cfg.hcolorbar, pcfg.colorbar = 1; end;
figh = [figh dc_plot(pcfg)];

pcfg = ccfg;
pcfg.traj = rbfqi(['replay datafile=' cfg.rbffile ' silent x0=[-pi,0] tend=1']);
pcfg.savefig = 'dcrbf_traj';
pcfg.xlim = []; pcfg.ylim = []; pcfg.figsize = [500 550];
pcfg.box = 0;
figh = [figh dc_plot(pcfg)];

% eval cost for RBF over coarse X0
if cfg.eval,
    load(cfg.rbffile, 'model');
    scfg.X0 = dc_problem('X0', cfg.X0);
    scfg.maxr = model.maxr;
    scfg.gamma = model.gamma;
    scfg.eps = cfg.mceps;
    scfg.datafile = cfg.rbffile;
    [J, R] = mc_rbfqi(scfg);
    RBFscore = J
end;    % last: result RBFscore = -210.7438

cd(crtdir)
% END dc_fzvsrbf
