function figh = dccons_lin(cfg)
% Consistency study of fuzzyqi for the DC motor
% Used for :
%   FUZZIEEE08 
%   Fuzzy Q-iter jnl paper draft
%   Ch4 of thesis and book

CFG.experiments = {'shapbox'};
% To plot ALL on-screen
% CFG.experiments = {'lqr', 'shaping', 'shapbox', 'BigBox', 'box'};
% % To generate plots for FUZZ-IEEE drafts 1, 1.5, 2
% CFG.plottarget = 'latex';
% CFG.experiments = {'lqr', 'shaping', 'shapbox'};
% To generate plots for FUZZ-IEEE draft 2
% CFG.plottarget = 'latex';
% CFG.experiments = {'lqr', 'shapbox'};

% cfg.N = 3:18;           % "quick" numbers
% cfg.M = 3:2:7;
CFG.N = 3:1:41;       % more detailed res study
CFG.M = 3:2:15;

CFG.datadir = 'd:\work\m\approxrl\systems\dcmotor\dccons_lin\';
CFG.savedir = 'D:\Work\tex\papers\fuzzieee08\img\';
CFG.problem = 'dc_problem';
CFG.namingstyle = 'old';        % 'old' = old style, compatible up to thesis and fzqi jnl papr
                                % 'new' = new style, starting Aug 2009,
                                % relies on prefix
CFG.prefix = 'dc_cons_fzqi';
CFG.X0 = 'coarse';
CFG.mc_eps = .1;
CFG.plottarget = 'screen';      % use 'screen' to not overwrite figures
CFG.plot3fun = @mesh;
CFG.view = {-21, 30};
CFG.figsize = [500 300];

if nargin < 1, cfg = struct; end;
if ischar(cfg),  cfg = str2cfg(cfg, fieldnames(CFG)); end;
cfg = checkparams(cfg, CFG);
cfg.grayscale = grayscalefromconfig(cfg);
cfg.xlim = [cfg.N(1) cfg.N(end)];
cfg.ylim = [cfg.M(1) cfg.M(end)];

if ischar(cfg.experiments), cfg.experiments = {cfg.experiments}; end;

% save crt dir, change to datadir
crtdir = pwd;
cd(cfg.datadir);

figh = [];

% main reward plot config
rpcfg.rew = 1;
rpcfg.savedir = cfg.savedir;
% rpcfg.savefig = 'lqr_rew';
rpcfg.plottarget = cfg.plottarget;
rpcfg.slice = [NaN, NaN, 0];      % 0-action slice
rpcfg.gridres = 60;
rpcfg.zlim = [-80 20];
rpcfg.view = cfg.view;
rpcfg.figsize = cfg.figsize;

% LQR basic options are needed as a baseline by other experiments
lqrcfg = cfg;
lqrcfg.model_params = {'lqr', 'lin'};  % default Q and R
lqrcfg.dfinfix = 'lqr';

% ------------------
% LQR reward experiments
if any(strcmp(cfg.experiments, 'lqr')),    
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'lqr', 'lin'};   % default Q and R
    if cfg.namingstyle(1) == 'o',   rewcfg.savefig = 'lqr_rew';
    else                            rewcfg.savefig = [cfg.prefix 'lqr_rew'];
    end;
    rewcfg.gridres = 60;
    rewcfg.zlim = [-80 10];
    figh = [figh dc_plot(rewcfg)];

    % run
    rcfg = lqrcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    % plot perf
    scfg = lqrcfg; 
    scfg.datafile = 'score_lqr';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_lqr';
    else                            scfg.savefig = [cfg.prefix 'lqr_perf'];
    end;
    scfg.zlim = [-400 -190];
    figh = [figh qiresstudy(scfg)];
    
   % zoom-in on perf
    scfg.N = 10:41;
    scfg.xlim = [10 41];
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_lqr_zoomin';
    else                            scfg.savefig = [cfg.prefix 'lqr_perf_zoomin'];
    end;
    scfg.zlim = [-207 -203];
    figh = [figh qiresstudy(scfg)];
    
end;

% ------------------
% Box shaping reward experiments
if any(strcmp(cfg.experiments, 'shapbox')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'rewtype=shapbox cshap=10', 'lin'};
    if cfg.namingstyle(1) == 'o',   rewcfg.savefig = 'shapbox_rew';
    else                            rewcfg.savefig = [cfg.prefix 'shapbox_rew'];
    end;
    rewcfg.gridres = 60;
    rewcfg.zlim = [-80 10];
    rewcfg.enhance = {[-pi/4 pi/4], [-4*pi 4*pi], []};

%     rewcfg.enhfactor = 4;
%     rewcfg.restrictx = [pi/4+pi/12, 4*pi+pi/3];
    figh = [figh dc_plot(rewcfg)];

    shapboxcfg = cfg;
    shapboxcfg.model_params = {'rewtype=shapbox cshap=10', 'lin'};
    shapboxcfg.dfinfix = 'shapbox';
    
    % run
    rcfg = shapboxcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ LQR reward fun
    scfg = shapboxcfg; 
    scfg.datafile = 'score_shapbox_lqr';
    scfg.perf = 1; scfg.userew = 'lqr';
    scfg.plottarget = cfg.plottarget;
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_shapbox_lqr';
    else                            scfg.savefig = [cfg.prefix 'shapbox_perf_lqrrew'];
    end;
    scfg.zlim = [-400 -190];
    figh = [figh qiresstudy(scfg)];
    
    % plot perf w/ original rew fun
    scfg = shapboxcfg; 
    scfg.datafile = 'score_shapbox';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_shapbox';
    else                            scfg.savefig = [cfg.prefix 'shapbox_perf_shapboxrew'];
    end;
    scfg.zlim = [-400 -150];   
    figh = [figh qiresstudy(scfg)];

    % 2-D comparison
    scfg = lqrcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.addtocrt = 0; scfg.legendpostfix = ', quadratic reward';
    scfg.lsty = '-';
    scfg.plottarget = cfg.plottarget;
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_lqrvsshapbox';
    else                            scfg.savefig = [cfg.prefix 'lqrvsshapbox_perf'];
    end;
    qiresstudy(scfg);

    clear scfg;
    scfg = shapboxcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_shapbox_lqr';
    scfg.userew = 'lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.lsty = '--';
    scfg.markertype = 's';
    scfg.addtocrt = 1; scfg.legendpostfix = ', discontinuous reward';
    scfg.plottarget = cfg.plottarget;
    if cfg.namingstyle(1) == 'o',   scfg.savefig = 'score_lqrvsshapbox';
    else                            scfg.savefig = [cfg.prefix 'lqrvsshapbox_perf'];
    end;
    figh = [figh qiresstudy(scfg)];
    
end;


% ------------------
% LQR+shaping reward experiments
% WARNING new style naming not implemented for this option
if any(strcmp(cfg.experiments, 'shaping')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'shaping', 'lin'};
    rewcfg.savefig = 'shaping_rew';
    rewcfg.enhance = {[-pi/4 pi/4], [-4*pi 4*pi], []};
    figh = [figh dc_plot(rewcfg)];

    shapcfg = cfg;
    shapcfg.model_params = {'shaping', 'lin'};
    shapcfg.dfinfix = 'shaping';
    
    % run
    rcfg = shapcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ LQR reward fun
    scfg = shapcfg; 
    scfg.datafile = 'score_shaping_lqr';
    scfg.perf = 1; scfg.userew = 'lqr';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_shaping_lqr';
    scfg.zlim = [-400 -190];
    figh = [figh qiresstudy(scfg)];
    
    % plot perf w/ original rew fun
    scfg = shapcfg; 
    scfg.datafile = 'score_shaping';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_shaping';
    scfg.zlim = [-400 -150];     % auto
    figh = [figh qiresstudy(scfg)];

    % 2-D comparison
    scfg = lqrcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.addtocrt = 0; scfg.legendpostfix = 'quadratic reward';
    scfg.lsty = '-';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshaping';
    qiresstudy(scfg);

    clear scfg;
    scfg = shapcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_shaping_lqr';
    scfg.userew = 'lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.lsty = '-.';
    scfg.addtocrt = 1; scfg.legendpostfix = 'discontinuous reward';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshaping';
    figh = [figh qiresstudy(scfg)];

end;


% ------------------
% Box shaping reward experiments
% WARNING new style naming not implemented for this option
if any(strcmp(cfg.experiments, 'shapboxc1')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'rewtype=shapbox cshap=1', 'lin'};
    rewcfg.savefig = 'shapboxc1_rew';
    rewcfg.enhance = {[-pi/4 pi/4], [-4*pi 4*pi], []};
    figh = [figh dc_plot(rewcfg)];

    shapboxcfg = cfg;
    shapboxcfg.model_params = {'rewtype=shapbox cshap=1', 'lin'};
    shapboxcfg.dfinfix = 'shapboxc1';
    
    % run
    rcfg = shapboxcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ LQR reward fun
    scfg = shapboxcfg; 
    scfg.datafile = 'score_shapboxc1_lqr';
    scfg.perf = 1; scfg.userew = 'lqr';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_shapboxc1_lqr';
    scfg.zlim = [-400 -190];
    figh = [figh qiresstudy(scfg)];

    % zoom-in on perf
    scfg.N = 10:41;
    scfg.xlim = [10 41];
    scfg.savefig = 'score_shapboxc1_lqr_zoomin';
    scfg.zlim = [-207 -203];
    figh = [figh qiresstudy(scfg)];
    
    % plot perf w/ original rew fun
    scfg = shapboxcfg; 
    scfg.datafile = 'score_shapboxc1';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_shapboxc1';
    scfg.zlim = [-400 -150];   
    figh = [figh qiresstudy(scfg)];

    % 2-D comparison
    scfg = lqrcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.addtocrt = 0; scfg.legendpostfix = 'quadratic reward';
    scfg.lsty = '-';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshapboxc1';
    qiresstudy(scfg);

    clear scfg;
    scfg = shapboxcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_shapboxc1_lqr';
    scfg.userew = 'lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.lsty = '-.';
    scfg.addtocrt = 1; scfg.legendpostfix = 'discontinuous reward';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshapboxc1';
    figh = [figh qiresstudy(scfg)];
    
end;

% ------------------
% BigBox reward experiments
% WARNING new style naming not implemented for this option
if any(strcmp(cfg.experiments, 'BigBox')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'rewtype=box zeroband=[15*pi/180;0.4]', 'lin'};
    rewcfg.savefig = 'bigbox_rew';
    rewcfg.restrictx = [.6, .6];
    rewcfg.zlim = [0, 1]; 
    figh = [figh dc_plot(rewcfg)];

    bixboxcfg = cfg;
    bixboxcfg.model_params = {'rewtype=box zeroband=[15*pi/180;0.4]', 'lin'};
    bixboxcfg.dfinfix = 'BigBox';
    
    % run
    rcfg = bixboxcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ original rew fun
    scfg = bixboxcfg; 
    scfg.datafile = 'score_bigbox';
    scfg.perf = 1;
    scfg.mc_eps = 0.01;     % need better score evaluation precision, here rewards are in [0, 1]
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_bigbox';
    scfg.zlim = [];     % auto
    figh = [figh qiresstudy(scfg)];

    % plot perf w/ LQR reward fun
    scfg = bixboxcfg;     
    scfg.datafile = 'score_bigbox_lqr';
    scfg.perf = 1; scfg.userew = 'lqr';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_bigbox_lqr';
%     scfg.zlim = [-240 -220];
    figh = [figh qiresstudy(scfg)];

	% 2-D comparison
    scfg = lqrcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.addtocrt = 0; scfg.legendpostfix = 'quadratic reward';
    scfg.lsty = '-';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsbigbox';
    qiresstudy(scfg);

    clear scfg;
    scfg = bixboxcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_bigbox_lqr';
    scfg.userew = 'lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.lsty = '-.';
    scfg.addtocrt = 1; scfg.legendpostfix = 'discontinuous reward';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsbigbox';
    figh = [figh qiresstudy(scfg)];

end;

% ------------------
% box reward experiments
% WARNING new style naming not implemented for this option
if any(strcmp(cfg.experiments, 'box')),
    
    % restricted features: only w/ original reward

    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'rewtype=box zeroband=[5*pi/180;0.2]', 'lin'};
    rewcfg.savefig = 'box_rew';
    rewcfg.restrictx = [.6, .6];
    rewcfg.zlim = [0, 1]; 
    figh = [figh dc_plot(rewcfg)];
    
    boxcfg = cfg;
    boxcfg.model_params = {'rewtype=box zeroband=[5*pi/180;0.2]', 'lin'};
    boxcfg.dfinfix = 'box';
    boxcfg.mc_eps = 0.01;     % need better score evaluation precision, here rewards are in [0, 1]
    
    % run
    rcfg = boxcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ original rew fun
    scfg = boxcfg; 
    scfg.datafile = 'score_box';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_box';
    scfg.zlim = [];     % auto
    figh = [figh qiresstudy(scfg)];

end;


% change dir to orig
cd(crtdir)
    
