function lspidc_hcon(exp, mode, c)
% DC motor
% a) Explicitly parameterized policies vs. exact policy improvements
% b) Exact policy improvements vs. policy improvement with monotonicity constraints

% Currently available experiments:
% RBF ORTHO set with large # samples
%     --------- with small # samples
% RBF DISC with large # samples
% TRIANG ORTHO set with large # samples
%       ---    with small # samples
% TRIANG (DISC) with large # samples

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% main cfg
cfg = struct;
cfg.(mode) = 1;
cfg.gamma = .97;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
cfg.datadir = pwd;
% sample config is constant
cfg.adaptiveNs = 1;
% ##### First set of experiments: Ns = 5K, hNs = K; *** NOTE: ONLY RAN FOR ORTHO APPROXIMATORS! ***
% cfg.Ns = 5000; % sample generation method will differ according to experiment
% cfg.hmethod = 'rand'; cfg.hNs = 1000;
% ##### Second set of experiments: Ns = 10K, hNs = 5K;
cfg.Ns = 10000;  % sample generation method will differ according to experiment   
cfg.hmethod = 'rand'; cfg.hNs = 5000;
cfg.xspace = 'eq';
cfg.maxiter  = 50;
cfg.startat = 1;cfg.repeat = 5;
% perf eval
cfg.perf = 1;
% cfg.forcerecompute = 1;
cfg.forceeval = 0;                  % to evaluate performance even for non-convergent experiments
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
cfg.show = {'rep', 'gamma'};
% replay
cfg.x0 = [pi;0]; cfg.tend = 1.5;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;
% plot 
% if strcmp(mode, 'plot'), 
%     cfg.stats = 1; cfg.plot = {'J', 'iter'};
%     cfg.savefig = 'dch'; 
% %     cfg.savedir = 'D:\Work\tex\reports\2008-03-17_3wr\img\';
%     cfg.savedir = 'd:\work\tex\papers\lspionlcon\img\';
%     cfg.plottarget = 'screen'; 
%     cfg.xlim = [3 5]; cfg.xtick = 3:5;
%     cfg.figsize = [450 350];
%     cfg.ylim_matrix = [-314 -297; 0 35];
%     cfg.markerspace = 1;
%     % these limits need to be updated...
%     if strcmp(exp, 'r*'),
%         cfg.ylim_matrix = [-320 -298; 0 13];
%         cfg.markerspace = 0.5;
%     else
%         cfg.ylim_matrix = [-320 -298; 0 60];
%         cfg.markerspace = 3;
%     end;
% end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% ===========================
% Specialized configs
% exact policy improvement config
excfg = cfg;
excfg.alg = 'lspi';
excfg.infix = 'exact';

% explicit policy parameterization
hcfg = cfg;
hcfg.alg = 'lspih';
hcfg.infix = 'nilc';

% explicit policy parameterization with monotonicity constraints
hconcfg = cfg;
hconcfg.alg = 'lspih';
hconcfg.infix = 'mon';
hconcfg.con = 'type=mon dir=[-1,-1]';

% ===========================
% RBF ORTHO experiments
% exact himprove
expout = []; expout2 = [];        % reset figure additions
roexp = excfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro';
roexp.N        = 9; roexp.M = [2 3 4]; 
roexp.method   = 'rand';
roexp.addtocrt = 0; roexp.expprefix = 'exact h,';     % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;
if checkrunexp(roexp.miscinfo, exp) && strcmp(mode, 'plot'), expout2 = lspiexp(roexp); end; % create second set of figures for hc
% parameterized policy 
rohexp = hcfg;
rohexp.type     = 'rbfortho'; rohexp.miscinfo = 'roh';
rohexp.N        = 9; rohexp.M = [2 3 4]; 
rohexp.method   = 'rand';
rohexp.htype    = 'rbfsing';
rohexp.usesamples = roexp;
rohexp.addtocrt = expout; rohexp.expprefix = 'approximate h,';     % only effective in stats+plot mode
if checkrunexp(rohexp.miscinfo, exp), expout = lspiexp(rohexp); end;
% monotonous constraints
rohcexp = hconcfg;
rohcexp.type     = 'rbfortho'; rohcexp.miscinfo = 'rohc';
rohcexp.N        = 9; rohcexp.M = [2 3 4];  
rohcexp.method   = 'rand';
rohcexp.htype    = 'rbfsing';
rohcexp.usesamples = rohexp;
rohcexp.addtocrt = expout2; rohcexp.expprefix = 'constrained h,';   % only effective in stats+plot mode
if checkrunexp(rohcexp.miscinfo, exp), expout = lspiexp(rohcexp); end;


% ===========================
% TRIANG ORTHO experiments
% exact himprove
% uses RBF ORTHO samples above
expout = []; expout2 = [];       % reset figure additions
toexp = excfg;
toexp.type     = 'triangortho'; toexp.miscinfo = 'to';
toexp.N        = 9; toexp.M = [2 3 4]; 
toexp.method   = 'rand';
toexp.usesamples = roexp;
toexp.addtocrt = 0; toexp.expprefix = 'exact h,';     % only effective in stats+plot mode
if checkrunexp(toexp.miscinfo, exp), expout = lspiexp(toexp); end;
if checkrunexp(toexp.miscinfo, exp) && strcmp(mode, 'plot'), expout2 = lspiexp(toexp); end; % create second set of figures for hc
% parameterized policy 
tohexp = hcfg;
tohexp.type     = 'triangortho'; tohexp.miscinfo = 'toh';
tohexp.N        = 9; tohexp.M = [2 3 4]; 
tohexp.method   = 'rand';
tohexp.htype    = 'triangsing';
tohexp.usesamples = rohexp;
tohexp.addtocrt = expout; tohexp.expprefix = 'approximate h,';     % only effective in stats+plot mode
if checkrunexp(tohexp.miscinfo, exp), expout = lspiexp(tohexp); end;
% monotonous constraints
tohcexp = hconcfg;
tohcexp.type     = 'triangortho'; tohcexp.miscinfo = 'tohc';
tohcexp.N        = 9; tohcexp.M = [2 3 4]; 
tohcexp.method   = 'rand';
tohcexp.htype    = 'triangsing';
tohcexp.usesamples = rohcexp;
tohcexp.addtocrt = expout2; tohcexp.expprefix = 'constrained h,';   % only effective in stats+plot mode
if checkrunexp(tohcexp.miscinfo, exp), expout = lspiexp(tohcexp); end;



% ===========================
% RBF DISCRETE experiments NOTE NOT RUN FOR SMALLER # OF SAMPLES
% exact himprove
expout = []; expout2 = [];        % reset figure additions
rdexp = excfg;
rdexp.type     = 'rbfdisc'; rdexp.miscinfo = 'rd';
rdexp.method   = 'randdisc';
rdexp.N        = 9; rdexp.M = [3 5]; 
rdexp.addtocrt = 0; rdexp.expprefix = 'exact h,';     % only effective in stats+plot mode
if checkrunexp(rdexp.miscinfo, exp), expout = lspiexp(rdexp); end;
if checkrunexp(rdexp.miscinfo, exp) && strcmp(mode, 'plot'), expout2 = lspiexp(rdexp); end; % create second set of figures for hc
% parameterized policy 
rdhexp = hcfg;
rdhexp.type     = 'rbfdisc'; rdhexp.miscinfo = 'rdh';
rdhexp.N        = 9; rdhexp.M = [3 5];
rdhexp.method   = 'randdisc';
rdhexp.htype    = 'rbfsing';
rdhexp.usesamples = rdexp;
rdhexp.addtocrt = expout; rdhexp.expprefix = 'approximate h,';     % only effective in stats+plot mode
if checkrunexp(rdhexp.miscinfo, exp), expout = lspiexp(rdhexp); end;
% monotonous constraints
rdhcexp = hconcfg;
rdhcexp.type     = 'rbfdisc'; rdhcexp.miscinfo = 'rdhc';
rdhcexp.N        = 9; rdhcexp.M = [3 5];  
rdhcexp.method   = 'randdisc';
rdhcexp.htype    = 'rbfsing';
rdhcexp.usesamples = rdhexp;
rdhcexp.addtocrt = expout2; rdhcexp.expprefix = 'constrained h,';  % only effective in stats+plot mode
if checkrunexp(rdhcexp.miscinfo, exp), expout = lspiexp(rdhcexp); end;

% ===========================
% TRIANG DISC experiments NOTE NOT RUN FOR SMALLER # OF SAMPLES
% Uses RBF DISC samples above
% exact himprove
expout = []; expout2 = [];       % reset figure additions
texp = excfg;
texp.type     = 'triang'; texp.miscinfo = 'td';
texp.N        = 9; texp.M = [3 5]; 
texp.method   = 'randdisc';
texp.usesamples = rdexp;
texp.addtocrt = 0; texp.expprefix = 'exact h,';     % only effective in stats+plot mode
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;
if checkrunexp(texp.miscinfo, exp) && strcmp(mode, 'plot'), expout2 = lspiexp(texp); end; % create second set of figures for hc
% parameterized policy 
thexp = hcfg;
thexp.type     = 'triang'; thexp.miscinfo = 'tdh';
thexp.N        = 9; thexp.M = [3 5]; 
thexp.method   = 'randdisc';
thexp.htype    = 'triangsing';
thexp.usesamples = rdhexp;
thexp.addtocrt = expout; thexp.expprefix = 'approximate h,';     % only effective in stats+plot mode
if checkrunexp(thexp.miscinfo, exp), expout = lspiexp(thexp); end;
% monotnous constraints
thcexp = hconcfg;
thcexp.type     = 'triang'; thcexp.miscinfo = 'tdhc';
thcexp.N        = 9; thcexp.M = [3 5]; 
thcexp.method   = 'randdisc';
thcexp.htype    = 'triangsing';
thcexp.usesamples = rdhcexp;
thcexp.addtocrt = expout2; thcexp.expprefix = 'constrained h,';   % only effective in stats+plot mode
if checkrunexp(thcexp.miscinfo, exp), expout = lspiexp(thcexp); end;

