% Script to plot 2-D composition (pyramids) of 1-D triangular part
% Harcoded for a 3-level partition for the DD problem
%
% Config adapted for CEPS paper

cfg.samefig = 0;
cfg.plottarget = 'latex';
cfg.savedir = 'D:\Work\tex\papers\ceps\img\';
cfg.savefig = 'dd_mfs';
cfg.figsize = [300 300];

% init
C1 = [-1 0 1]; C2 = [-0.5 0 0.5];
mfs1 = gen_mfs(C1); mfs2 = gen_mfs(C2);
C1x = [C1(1) C1 C1(end)];
C2x = [C2(1) C2 C2(end)];

lsty = {{'k-'}, {'k:'}, {'k-.'}};
commonprop = {'Interpreter','LaTeX','FontSize',11};

% plot

% % Prepare figure 1
% if cfg.samefig, 
%     figh(end+1) = figurex(cfg.figsize);
%     subplot(121); 
% else
%     figh(end+1) = figurex(cfg.figsize - [50 120]);
% end;
% 
% % -------------------------
% % Figure 1: 1-D MFs
% 
% for i = 1:length(C),
%     % compute the vector of values of MF#i in all the points in domain #i
%     d = Cx(i:i+2); mu = 0*d;
%     for j = 2:length(d-1),
%         m = mdegs(d(j), mfs);
%         mu(j) = m(i);
%     end;
%     mu(1) = 0; mu(end) = 0;
%     plot(d, mu, lsty{i}{:}, 'LineWidth', 2); hold on;
% end;
% xlabel('DOTCX'); 
% ylabel('MUDOTCX');
% 
% % Close figure 1, prepare figure 2
% if cfg.samefig, subplot(122); 
% else
% %     saveplot(figh(end), [cfg.savedir cfg.onedimfig], cfg.plottarget);
%     figh(end+1) = figurex(cfg.figsize);
% end;

% -------------------------
% Figure 2: 2-D composition
figh = figurex(cfg.figsize);
colormap(gray);
Np = 14;
eint = [0, .8, .3;
       .6, 0, .6; 
       .3, .8, 0];
fint = [.8, .5, .8; 
        .8, .3, .8;
        .8, .5, .8];
for i = 1:length(C1),
    for j =1:length(C2),
        % 2-D domain of composite MF i,j
        d = [C1x(i) : (C1x(i+2)-C1x(i))/Np : C1x(i+2)
             C2x(j) : (C2x(j+2)-C2x(j))/Np : C2x(j+2)];
        mu = d*0;
        for i1 = 1:size(d, 2),
            m1 = mdegs(d(1, i1), mfs1); m1 = m1(i);
            for j1 = 1:size(d, 2),
                m2 = mdegs(d(2, j1), mfs2); m2 = m2(j);
                mu(i1, j1) = m1 * m2;
            end;
        end;
        mesh(d(1, :), d(2, :), mu', 'EdgeColor', eint(i,j)*[1 1 1], ...
            'FaceColor', fint(i,j)*[1 1 1], 'FaceAlpha', 1, 'EdgeAlpha', 1); hold on;
    end;
end;

% % draw edges
% mux1 = 0*Cx; mux2 = 0*C; 
% mux1(2:2:end-1) = 1;
% mux2(2:2:end-1) = 1;
% esty = {'LineWidth', 2, 'Color', 'k', 'LineStyle', '-'};
% line(Cx, -2+0*Cx, mux1, esty{:}); line(C, -2+0*C, mux2, esty{:});
% line(-2+0*Cx, Cx, mux1, esty{:}); line(-2+0*C, C, mux2, esty{:});

xlabel('$x_1$', commonprop{:});
ylabel('$x_2$', commonprop{:}); 
zlabel('$\varphi_i(x_1, x_2)$', commonprop{:}); 
% no-latex version
% xlabel('x_1');
% ylabel('x_2'); 
% zlabel('\phi_i(x_1, x_2)'); 


% if cfg.samefig, 
%     saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
% else
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
% end;

% pause and close
close(figh);