%% PID Controller Design

hwinit;
joint = [5];                                % index of joint(s) to be controlled
                                            % (1 = base, 2 = shoulder, etc.)
                                            % can be a vector too

%% Constants, flags and file names
h  = 0.01;                      % sampling time [s]
simflag = 0;                    % set to 1 for simulation, 0 for real time
simfile = 'robot_pid';          % simulated controller (Simulink)   
rtfile = 'rtrobot_pid';         % real-time controller (Simulink)   

%% Take care of different MATLAB versions
mv = version;                   % check MTB version
mv = str2num(mv(1));

if simflag,                     
    file = simfile;             % use simulation
else
    file = rtfile;              % use real-time
end;

if mv == 6,                     
    file = [file '_v6'];        % file for Matlab 6.x
end;    

% from step response
tau = 0.15;
km = 5;

tau = 0.06;             % wrist
km = 5;
tau = 0.025;           % wrist rotation
km = 4.5;

%% Design parameters
w = 12;
z = .9;
refmod = tf(w^2,[1 2*z*w w^2]);

%% Compute controller gains

Kp = tau*w^2/km;
Kd = (2*z*w*tau - 1)/km;
Ki = 0;

Kp = 1.5;
Kd = 0.01;
Ki = 0;

return;

%% Run control experiment
sim(file);

%% Compare with model
yref = lsim(refmod,r,t);        % simulate the reference model
figure(1); clf;
plot(t,r,t,yref,t,y);           % compare model with real time
title('Closed-loop response');
xlabel('Time [s]'); 
ylabel('Reference and actual angle [rad]')

figure(2); clf;
plot(t,u);                      % plot u
title('Closed-loop response');
xlabel('Time [s]'); 
ylabel('Control input [-]')

