clear all

% What experiments to process and in what mode
runids = '*hf';
mode = 'cleanup';       % 'run', 'sol', 'stats', 'cleanup'

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.problem = 'invpend_problem';
cfg.model_params = {'lqr', 'rot', 'Ts=0.02'};
cfg.samesamples = 1; 
cfg.sstest = {'Ns'}; % THIS IS NOT TESTED -- but only active in run mode
% these are all common settings
cfg.gamma = [.95:.01:.99 .992:.002:.998];
cfg.N = 9; cfg.M = 3; cfg.Ns = 10000;
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.maxiter  = 200;
cfg.repeat = 5;
% perf eval
cfg.perf = 0;                       % turn off for now, we care about iter only
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
cfg.x0 = [pi;0]; cfg.tend = 1.5;    % for replaying
cfg.datadir = 'd:\work\m\approxrl\systems\invpend\lspiip_gammastudy';
% customize the plots
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'iter'}; 
    cfg.savefig = 'ipgamma';
    cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
%     cfg.plottarget = 'latex'; 
    cfg.xtick = [.95:.01:.99 .998];
    cfg.xlim = [.95 .998];
    cfg.ylim = [0 200];
    cfg.figsize = [400 300];
    cfg.markerspace = 5;
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% prepare initial policies for h0 experiments
cfg_hf = cfg;
% cfg_hf.gamma = [.95 .97 .99 .994 .998];  % subset, comment out to use the entire range as above
cfg_hf.samesamples = 0; % instead, use samples from the no-initial policy experiments
h0 = {};
h0args = {};
fzdir = 'd:\work\m\approxrl\systems\invpend\fzip_gammastudy\';
for i = 1:length(cfg_hf.gamma),
    h0{i} = @fuzzyqi_h;
    h0args{i} = {fuzzyqi_prepareh([fzdir 'fzip_fine_g' num2strx(cfg_hf.gamma(i))], 0)};  % 0 = no interpolation;
end;
cfg_hf.lspi_params = struct('h0', h0, 'h0args', h0args);
cfg_hf.infix = 'hf';  
cfg_hf.repeat = 2;

% -----------------------
% No prior policy
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't';
texp.method = 'randdisc';
if checkrunexp(texp.miscinfo, runids), texpout = lspiexp(texp); end;
%
tqexp = cfg;
tqexp.type     = 'triangquad'; tqexp.miscinfo = 'tq';
tqexp.method = 'rand';
if checkrunexp(tqexp.miscinfo, runids), tqexpout = lspiexp(tqexp); end;

rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r';
rexp.method = 'randdisc';
if checkrunexp(rexp.miscinfo, runids), rexpout = lspiexp(rexp); end;
%
rqexp = cfg;
rqexp.type     = 'rbfquad'; rqexp.miscinfo = 'rq';
rqexp.method = 'rand';
if checkrunexp(rqexp.miscinfo, runids), rqexpout = lspiexp(rqexp); end;

% ----------------------
% Near-optimal policy computed with fuzzy QI
texp_hf = cfg_hf;
texp_hf.type     = 'triang'; texp_hf.miscinfo = 'thf';
texp_hf.method = 'randdisc';
texp_hf.usesamples = texp;
if checkrunexp(texp_hf.miscinfo, runids), 
    if strcmp(mode, 'plot'), texp_hf.addtocrt = texpout; texp_hf.expprefix = 'h_0=~h*,'; end;
    exp_hfout = lspiexp(texp_hf); 
end;
%
tqexp_hf = cfg_hf;
tqexp_hf.type     = 'triangquad'; tqexp_hf.miscinfo = 'tqhf';
tqexp_hf.method = 'rand';
tqexp_hf.usesamples = tqexp;
if checkrunexp(tqexp_hf.miscinfo, runids), 
    if strcmp(mode, 'plot'), tqexp_hf.addtocrt = tqexpout; tqexp_hf.expprefix = 'h_0=~h*,'; end;
    exp_hfout = lspiexp(tqexp_hf); 
end;

rexp_hf = cfg_hf;
rexp_hf.type     = 'rbfdisc'; rexp_hf.miscinfo = 'rhf';
rexp_hf.method = 'randdisc';
rexp_hf.usesamples = rexp;
if strcmp(mode, 'plot'), rexp_hf.addtocrt = rexpout; rexp_hf.expprefix = 'h_0=~h*,'; end;
if checkrunexp(rexp_hf.miscinfo, runids), exp_hfout = lspiexp(rexp_hf); end;
%
rqexp_hf = cfg_hf;
rqexp_hf.type     = 'rbfquad'; rqexp_hf.miscinfo = 'rqhf';
rqexp_hf.method = 'rand';
rqexp_hf.usesamples = rqexp;
if strcmp(mode, 'plot'), rqexp_hf.addtocrt = rqexpout; rqexp_hf.expprefix = 'h_0=~h*,'; end;
if checkrunexp(rqexp_hf.miscinfo, runids), exp_hfout = lspiexp(rqexp_hf); end;
