global LineWidthOrder;

% Ad-hoc script to plot value function and policy, and some trajectories on top of them
cfg.X0 = [pi 0; -pi 0.6; 0 12*pi; 0 -12*pi; 1 -20; -1 20]';
cfg.userinput = 0;
cfg.style = 'arrows';
cfg.subsample = 4;

% Obtain value function
FH = fuzzyqi_prepareh('D:\Work\m\approxrl\systems\invpend_setup\fuzzyqi_ipsetup\ipsfz_fine_M41equid_lqr', 0);
[X, V] = fuzzyqi_v(FH); 

% plot value function
figure(10); clf; 
mh = mesh(X{1}, X{2}, V'); 
xlabel('Hoek \alpha', 'FontSize', 20); ylabel('Hoeksnelheid \alpha''', 'FontSize', 20); 
zlabel('Score', 'FontSize', 20); 
set(mh, 'FaceColor', 'interp', 'EdgeColor', 'none');
hold on;
movegui(gcf, 'west');
figure(11); clf; ph = pcolor(X{1}, X{2}, V'); set(ph, 'EdgeColor', 'none'); hold on;
movegui(gcf, 'east');

% plot trajectories
colors = {'b', 'g', 'm', 'y', 'r', 'c', 'k'};
i = 1; stopped = 0;
while ~stopped,
    % do a replay
    if i <= size(cfg.X0, 2),
        cfg.x0 = cfg.X0(:, i);
    else
        if ~cfg.userinput, stopped = 1; break; end;
        figure(11);
        [x, y, button] = ginput(1);
        if button > 1, stopped = 1; break; end;
        cfg.x0 = [x; y];
    end;
    cfg.tend = 3;
    cfg.datafile = 'D:\Work\m\approxrl\systems\invpend_setup\fuzzyqi_ipsetup\ipsfz_fine_M41equid_lqr';
    cfg.noplot = 1;
    cfg.verb = -Inf;
    [hist] = fuzzyqi_replay(cfg);
    K = length(hist.x);
    VX = zeros(1, K);
    for j = 1:K, VX(j) = fuzzyqi_v(hist.x(:, j), FH); end;
    % make a new color if needed
    if i <= length(colors),     c = colors{i};
    else
        switch cfg.style,
            case 'bullets',
                c = rand(3, 1);
            case 'arrows'
                c  = 'x';
        end;
    end;
    t = unique([1:cfg.subsample:K K]); % making sure K is there
    figure(10);
%     plot3(0, 0, fuzzyqi_v([0; 0], FH), 'Marker', '^', 'MarkerSize', 30, 'MarkerEdgeColor', 'k', 'LineWidth', 3);
    switch cfg.style,
        case 'bullets',
            plot3(hist.x(1, t), hist.x(2, t), VX(t), 'o', ...
                'MarkerEdgeColor','none', 'MarkerFaceColor', c, 'MarkerSize', 3);
            figure(11);
            plot(hist.x(1, t), hist.x(2, t), 'o', ...
                'MarkerEdgeColor','none', 'MarkerFaceColor', c, 'MarkerSize', 3);
        case 'arrows';
            % this is pretty complicated and requires fine tuning...
            xlim([min(X{1}), max(X{1})]);
            ylim([min(X{2}), max(X{2})]);
            zlim([-6000 0]);
            pbaspect([2 2 1]);
            view([160 42]);
            set(gcf, 'Position', [0 0 1000 750]);
            P1 = [hist.x(1, t(1:end-1))', hist.x(2, t(1:end-1))', VX(t(1:end-1))'];
            P2 = [hist.x(1, t(2:end))', hist.x(2, t(2:end))', VX(t(2:end))'];
            % check if there is a "jump" over the end of the interval
            kd = find(abs(diff(hist.x(1, t))) > 0.8 * 2*pi);
            % eliminate any corresponding arrow
            if ~isempty(kd),
                dp = P2(kd, :) - P1(kd, :);
                P1 = P1([1:kd-1 kd+1:end], :);
                P2 = P2([1:kd-1 kd+1:end], :);
            end;
            % make arrow height proportional to the distance between points
            H = sqrt(sum((P2 - P1).^2, 2)) ./ 75;
            % within limits
            H = max(min(H, 3), 0.7);
            % arrow width smaller than height, also, within limits
            W = max(min(H ./ 1.5, 1.5), 0.3);
            LineWidthOrder = 1 .* ones(size(H));
            qh3 = arrow3(P1, P2, [c '-/'], W, H); %5, ...
%                'a@', 'fancy', 'b@', 90);
%             set(qh, 'Marker', 'o', 'MarkerColor', c, 'MarkerFaceColor', c);
%             plot3(hist.x(1, :), hist.x(2, :), VX, 'o', ...
%                 'MarkerEdgeColor','none', 'MarkerFaceColor', c, 'MarkerSize', 3);
            figure(11);
    end;
    i = i + 1;
end;

% N = 30;
% X2 = -pi:2*pi/N:pi;
% Y2 = -15*pi:30*pi/(N+3):15*pi;
% [XX, YY] = meshgrid(X{1}, X{2});
% [XX2, YY2] = meshgrid(X2, Y2);
% V2 = interp2(XX, YY, V', XX2, YY2); % V2 = V2';
% [dVx, dVy] = gradient(V2, X2(2) - X2(1), Y2(2) - Y2(1));
% quiver(X2, Y2, dVy, dVx);
