function varargout = ipsetup_dummy_rtproblem(mode, sys, varargin)
% Dummy, false-real-time problem, actually runs simulation model.
% Useful to test real-time learning and control algorithms without actually using a real system.
% 
%   OUT = IPSETUP_DUMMY_RTPROBLEM(MODE, SYS, VARARGIN)
%

% remember x and last transition
persistent xcrt

varargout = {};
switch mode
    
    % read state
    case 'readx',
        varargout = {xcrt};
        
    % write action
    case 'writeu',
        % simulate transition and remember state 
        u = varargin{1};
        xcrt = sys.simulationfun(sys, xcrt, u);   % note we ignore reward and terminal flags
        varargout = {u};
    
    % create system structure
    case 'system',
        sys = ipsetup_problem('model');
        sys.id = 'ipsetup_dummy_rt'; 
        sys.problem = @ipsetup_dummy_rtproblem;
        % replace old, simulation transition fun by the eval transition
        sys.simulationfun = sys.fun;
        sys.fun = @ipsetup_dummy_evaltrans;
        % prepare output
        % sys
        varargout = {sys};
        
    % initialize system
    case 'init',
        % there is no real system
        % reset persistent variables, signaling they are "empty"
        xcrt = [];
        varargout = {sys};
        
    %  finalize
    case 'close', 
        % do nothing, this is not a real-life system
        % output system
        varargout = {sys};
        
    % reset system to an initial state
    case 'reset',
        reset = varargin{1};
        xcrt = ipsetup_problem('reset', sys, reset);
        varargout = {xcrt};
        
end;    % main 'mode' switch

end


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local functions
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Evaluate transition
% Compute reward and terminal flag
function [rplus, term] = ipsetup_dummy_evaltrans(sys, x, u, xplus)

rplus = ipsetup_reward(sys, x, u, xplus);
term = 0;   % task is continuing

end
