% Fittedqi with extra-trees on the invpend problem

%% #### General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'fittedqi';
BCFG.verb = 5;

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.gamma = .98;
ACFG.maxiter  = 500;
ACFG.regmethod = 'extratrees';
ACFG.singlereg = 1;
ACFG.trees_ntrees = 50;
ACFG.Xtest = 'auto';
ACFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\llrfqi_ip_etfqicomparison\';
ACFG.U = {[-3 0 3]};
ACFG.verb = 3;
% perf eval options
ACFG.mc_eps = .1;
ACFG.mc_N = 1;
ACFG.X0 = 'smallsym';

cd(ACFG.datadir);


%% #### Experiment 1: nmin=5, effect of #samples // grid sampling

% NOTE! If using this in a paper, need to run 10 experiments as well -- since tree construction is random
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsgrid';
bcfg.idfields = {'alg', 'problem', 'trees_ntrees', 'trees_nmin', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'samplesxgrids';

acfg = ACFG;
acfg.trees_nmin = 5;
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};
acfg.samples.ugrids = acfg.U;

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'score';

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);


%% #### Experiment 2: nmin=5, effect of knn and #samples // unif random sampling
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsrand';
bcfg.idfields = {'alg', 'problem', 'trees_ntrees', 'trees_k', 'trees_nmin', 'samplesmethod', 'samplesN'};
bcfg.repeat = 10;
bcfg.in = 'samplesN';

acfg = ACFG;
acfg.trees_nmin = 5;
acfg.samples.method = 'randdisc';
acfg.samples.ugrids = acfg.U;
acfg.samples.N = [15 31 51 71].^2 * 3;  

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'score';

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);


%% #### Experiment 3: nmin = 2, effect of #samples // grid sampling

% NOTE! If using this in a paper, need to run 10 experiments as well -- since tree construction is random
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsgrid';
bcfg.idfields = {'alg', 'problem', 'trees_ntrees', 'trees_nmin', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'samplesxgrids';

acfg = ACFG;
acfg.trees_nmin = 2;
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};
acfg.samples.ugrids = acfg.U;

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'iter';

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);


%% #### Experiment 4: nmin = 2, effect of knn and #samples // unif random sampling

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsrand';
bcfg.idfields = {'alg', 'problem', 'trees_ntrees', 'trees_k', 'trees_nmin', 'samplesmethod', 'samplesN'};
bcfg.repeat = 5;
bcfg.in = 'samplesN';

acfg = ACFG;
acfg.trees_nmin = 2;
acfg.samples.method = 'randdisc';
acfg.samples.ugrids = acfg.U;
acfg.samples.N = [15 31 51 71].^2 * 3;  

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'iter';

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);

