% LSPI on the invpend problem, for comparison with LLR fitted QI

%% #### General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'lspi';
BCFG.verb = 5;

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.gamma = .98;
ACFG.maxiter  = 200;
ACFG.eps = 0.1;     % the same value as the llrfittedqi default
ACFG.epsper = 0.1;  % same for checking oscillation
ACFG.approx.enhanced = 1;
ACFG.approx.type = 'triang';
ACFG.approx.N = [11 11]; ACFG.approx.xspace = 'eq';
ACFG.approx.M = 3; ACFG.approx.uspace = 'eq';
ACFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\llrfqi_ip_lspicomparison\';
ACFG.verb = 3;
% perf eval options
ACFG.mc_eps = .1;
ACFG.mc_N = 1;
ACFG.X0 = 'smallsym';

cd(ACFG.datadir);


%% #### Experiment 1: effect of #samples // grid sampling

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsgrid';
bcfg.idfields = {'alg', 'problem', 'approxtype', 'approxN', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'samplesxgrids';

acfg = ACFG;
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};
acfg.samples.ugrids = {[-3 0 3]};

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'score';

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);


%% #### Experiment 2: effect of knn and #samples // unif random sampling

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'Nsrand';
bcfg.idfields = {'alg', 'problem', 'approxtype', 'approxN', 'samplesmethod', 'samplesN'};
bcfg.repeat = 10;
bcfg.in = 'samplesN';

acfg = ACFG;
acfg.samples.method = 'randdisc';
acfg.samples.ugrids = {[-3 0 3]};
acfg.samples.N = [15 31 51 71].^2 * 3;  

%% Run
processbatch(bcfg, acfg);

%% Eval in own figure
bcfg.out = 'score';
bcfg.addtocrt = 0; bcfg.plotindex = 1;

acfg.samples.N = [31 51 71].^2 * 3;     % we don't bother with 15x15 since A is noninv for all those    

evalbatch(bcfg, acfg);

%% Eval adding to current
bcfg.out = 'score';

acfg.samples.N = [31 51 71].^2 * 3;     % we don't bother with 15x15 since A is noninv for all those    

bcfg.addtocrt = 1; bcfg.plotindex = 2;
evalbatch(bcfg, acfg);
