function lspeos_explordecay(exp, mode, cfg)
% Study of exploration for LSPE online/optimistic

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'choosealpha'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, cfg = struct; end;

% === DEFAULT CONFIG
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 20;

% LSPE algorithm config, partly via LSPIEXP meta-config fields 
% (including approx, problem, etc.)
CFG.alg = 'lspeonline';
CFG.problem = 'ipsetup_problem';
CFG.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
CFG.gamma = 0.98;
CFG.type = 'rbfdisc'; CFG.N = 11; CFG.M = 3; CFG.xspace = 'eq'; CFG.uspace = 'log';
CFG.dupd = .05;                             % i.e., once very 10 samples
CFG.trialtime = 1.5;                        % sufficient for stabilization 
CFG.lspi_params.maxtime  = 60*10;           % 10 simulated minutes
CFG.lspi_params.explor = 1;                 % start with this exploration rate
CFG.explordecay = [0.1 .^ (1 ./ [20 50 100 200 300 600])  0.5^(1/600)];
% stats config
CFG.lspi_params.dstoretheta = 1;
CFG.lspi_params.verb = 5;
CFG.lspi_params.ddisp = 10;
CFG.lspi_params.dsave = 60;

% LSPIEXP meta-config
% data files
CFG.datadir = pwd; CFG.show = {'explordecay', 'alpha', 'rep'}; CFG.hide = {'approx'};
% perf eval
CFG.X0 = 'small'; CFG.perf = 1; CFG.mc_eps = 0.1; 
CFG.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
CFG.forceeval = 1;                         % in STATS mode -- since no exp actually converges
% replay
CFG.x0 = [-pi;0]; CFG.tend = 3; 
% plot
CFG.savedir = '';   % dummy so that this value can be set by user
CFG.plottarget = 'screen'; CFG.savefig = 'ed';
CFG.plotconf = 1; CFG.plotminmax = 0; CFG.minmaxtype = 'area';
CFG.fixedfigname = 0;
CFG.addtocrt = [];
CFG.stats = 0;
CFG.plot = [];
CFG.perf = 0;
if any(strcmp(mode, {'onlineperf', 'onlineconv'})), CFG.verb = 0; else CFG.verb = 5; end;

% === INSTALL CUSTOM CONFIG SETTINGS
cfg = parseconfig(cfg, CFG);

expout = [];   % dummy


% === EXPERIMENT TO CHOOSE ALPHA
% These experiments are run on the new TU desktop computer
caexp = cfg;
caexp.miscinfo = 'choosealpha';
caexp.infix = 'choosealpha';
caexp.repeat = 3;       % do a few passes only (this is a preliminary experiment)
% test these values of alpha (stepsize)
caexp.alpha = [.001 .01 .1 1];
% use periodic resets to a fixed state, to also be able to 
% evaluate the "onlinereturn"
caexp.lspi_params.reset = 'randfixed';
caexp.lspi_params.x0 = [-pi; 0];
caexp.lspi_params.resetper = 10;

if checkrunexp(caexp.miscinfo, exp), 
    switch mode,
        % in plot modes, make separate plots for each value of explordecay
        case {'onlineperf', 'onlineconv'},
            ed = caexp.explordecay;
            for i = 1:length(ed),
                caexp.explordecay = ed(i); expout = lspiexp(caexp);
                title(['\epsilon_d=' sprintf('%.3f', ed(i))]);
                set(gcf, 'Name', sprintf('eps_d=%.3f', ed(i)));
            end;
        otherwise,
            expout = lspiexp(caexp); 
    end;
end;


% === STUDY OF EXPLORATION, FOR ALPHA CHOSEN ABOVE
% These experiments are run on the new TU desktop computer
edexp = cfg;
edexp.miscinfo = 'ed';
edexp.infix = 'ed';
% corresponding alpha values for each value of dupd (20 EXPERS AVAILABLE)
edexp.alpha = [NaN NaN NaN NaN NaN NaN];
% the values of alpha above should be used for the respective dupd experiments;
% do not run experiments for crossproduct of alpha and dupd:
edexp.donotexpand = {'alpha'};
if checkrunexp(edexp.miscinfo, exp), expout = lspiexp(edexp); end;


