function lspihs_h(exp, mode, c)
% Inv pend setup. Attempts with explicitly parameterized policies

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% main cfg
cfg = struct;
cfg.(mode) = 1.5;
cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.98;
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.datadir = pwd;
% sample config is constant
cfg.adaptiveNs = 1;
cfg.xspace = 'eq'; % cfg.uspace = 'log'; -- never used, we don't have disc approx currently
cfg.maxiter  = 50;
cfg.startat = 1; cfg.repeat = 5;
% perf eval
cfg.perf = 1;
cfg.forcerecompute = 0;
cfg.forceeval = 1;                  % to evaluate performance even for non-convergent experiments
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
cfg.show = {'rep'};
% replay
cfg.x0 = [-pi;0]; cfg.tend = 1.5;
% if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;
% plot 
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J', 'iter'};
    cfg.savefig = 'dch'; % cfg.savedir = 'D:\Work\tex\reports\2008-03-17_3wr\img\';
    cfg.plottarget = 'screen'; 
%     cfg.xlim = [3 5]; cfg.xtick = 3:5;
    cfg.figsize = [450 350]; 
end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% Specialized configs
% exact policy improvement config
excfg = cfg;
excfg.alg = 'lspi';
excfg.infix = 'exact';

% explicit policy parameterization
hcfg = cfg;
hcfg.alg = 'lspih';
hcfg.infix = 'nilc';

% ===========================
% RBF ORTHO experiments


% parameterized policy 
expout = [];
rohexp = hcfg;
rohexp.type     = 'rbfortho'; rohexp.miscinfo = 'roh';
rohexp.N        = 11; rohexp.M = [2]; % 3 4]; 
rohexp.method   = 'rand'; 
rohexp.htype    = 'rbfsing';
rohexp.hrbfrad  = 'avg';
rohexp.hmethod = 'rand'; 
% One experiment (#1) available for the following combinations:
% rohexp.Ns = 5000; rohexp.hN = 11; rohexp.hxspace  = 'eq'; rohexp.hNs = 1000;        % -2135.0 nonconv/osc
% rohexp.Ns = 5000; rohexp.hN = 19; rohexp.hxspace  = 'eq'; rohexp.hNs = 1000;        % -2165.6 nonconv/osc
% rohexp.Ns = 5000; rohexp.hN = 19; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;       % -2122.6 osc
% rohexp.Ns = 5000; rohexp.hN = 21; rohexp.hxspace  = 'log'; rohexp.hNs = 10000;      % -2170.1 nonconv
% rohexp.Ns = 10000; rohexp.hN = 25; rohexp.hxspace  = 'log'; rohexp.hNs = 10000;     % -2119.1 conv
% rohexp.Ns = 10000; rohexp.hN = 25; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;      % -2126.4 conv
% rohexp.Ns = 5000; rohexp.hN = 25; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;       % -2131.7 osc
% rohexp.Ns = 10000; rohexp.hN = 15; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;      % -2119.6 conv
% rohexp.Ns = 10000; rohexp.hN = 29; rohexp.hxspace  = 'eq'; rohexp.hNs = 15000;      % -2167.8 conv
% rohexp.Ns = 5000; rohexp.hN = 11; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;       % -2114.0 osc
% rohexp.Ns = 10000; rohexp.hN = 11; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;      % -2098.2 conv
% rohexp.Ns = 10000; rohexp.hN = 11; rohexp.hxspace  = 'eq'; rohexp.hNs = 5000;       % -2151.6 conv
% 
% TODO Run several experiments for this
rohexp.Ns = 10000; rohexp.hN = 11; rohexp.hxspace  = 'eq'; rohexp.hNs = 10000;
rohexp.addtocrt = 0; rohexp.expprefix = 'approximate h,';     % only effective in stats+plot mode
if checkrunexp(rohexp.miscinfo, exp), expout = lspiexp(rohexp); end;


% exact himprove
roexp = excfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro';
roexp.N        = 11; roexp.M = [2]; % 3 4]; 
roexp.method   = 'rand'; 
% One experiment (#1) available for the following combinations:
% roexp.Ns = 5000;      % -2106.3
%
% TODO Run several experiments for this
roexp.Ns = 10000;      % -2140.2
roexp.addtocrt = expout; roexp.expprefix = 'exact h,';     % only effective in stats+plot mode
roexp.usesamples = rohexp;
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

