function lspis_samptraj(exp, mode, c)
% Real inverted pendulum. Using trajectories for samples

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
cfg.(mode) = 2;
cfg.problem = 'ipsetup_problem';
% this should not give convergence problems, while still giving a time horizon for a good solution
cfg.gamma = 0.98;
cfg.maxiter  = 150;
cfg.repeat = 1; cfg.startat = 1;
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.adaptiveNs = 1;
cfg.Ns = 5000;         % baseline Ns
cfg.samesamples = 0;   % instead, load among experiments
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.datadir = pwd;
% perf eval
cfg.perf = 1; cfg.X0 = 'coarse'; cfg.mc_eps = 0.1;
% replay
cfg.x0 = [pi;0]; cfg.tend = 5;    % for replaying
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'}; 
    cfg.savefig = 'ipapprox';
%     cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
%     cfg.plottarget = 'latex'; 
    cfg.xlim = [2 4]; cfg.xtick = 2:4;
%     cfg.ylim_matrix = [-225 -210; 0 100];
    cfg.figsize = [450 350];
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% discrete samples generation config
discsamples_params.K = 250;
discsamples_params.policy = 'hrand';
discsamples_params.policyargs = {[-3 0 3], 1};

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);
expout = [];   % dummy

% % RBF poly
% rpexp = cfg;
% rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp';
% rpexp.N        = 11; rpexp.M = [2 3 4];
% rpexp.method   = 'rand';
% rpexp.usesamples = tpexp;
% rpexp.addtocrt = 0; rpexp.expprefix = 'poly,'; % only effective in stats+plot mode
% if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;

% RBF discrete
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r';
rexp.N        = 11; rexp.M = 3;
rexp.method   = 'randdisc';
rexp.addtocrt = expout; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% with trajectory sampling
rtexp = cfg;
rtexp.type     = 'rbfdisc'; rtexp.miscinfo = 'rt';
rtexp.N        = 11; rtexp.M = 3;
rtexp.method   = 'randtrajdisc';
rtexp.lspi_params.invdelta = 0.001;
rtexp.samples_params = discsamples_params;
rtexp.addtocrt = expout; rtexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rtexp.miscinfo, exp), expout = lspiexp(rtexp); end;

% with batch semi-online sampling
rbexp = cfg;
rbexp.alg      = 'lspibatch';
rbexp.type     = 'rbfdisc'; rbexp.miscinfo = 'rb';
rbexp.N        = 11; rbexp.M = 3;
rbexp.method   = 'randtrajdisc';
rbexp.samples_params = discsamples_params;
rbexp.Ns       = 1000;                  % start with a few samples only
rbexp.lspi_params.diter    = 4;         % and at every diter iterations
rbexp.lspi_params.dNs      = 1000;      % add dNs more
rbexp.lspi_params.Nsmax    = 7000;      % until at most Nsmax samples have accumulated
rbexp.lspi_params.invdelta = 0.001;
rbexp.lspi_params.dsamples.K = 200;     % trajectory length for sampling
rbexp.lspi_params.dsamples.method = 'randtrajdisc';
rbexp.lspi_params.dsamples.explor = 0.5;
rbexp.addtocrt = expout; rbexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rbexp.miscinfo, exp), expout = lspiexp(rbexp); end;

% % Orthogonal polynomial RBF -- only 1 experiment ran!
% roexp = cfg;
% roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro';
% roexp.N        = 11; roexp.M = [2 3]; roexp.orthotype = 'cheby';
% roexp.method   = 'rand';
% roexp.usesamples = tpexp;
% roexp.addtocrt = 0; roexp.expprefix = 'ortho,'; % only effective in stats+plot mode
% if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;


end



