%% Optimistic planning experiments for the inverted pendulum
function opd_detip(mode, expids, custombcfg, customacfg)

%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planalg', 'X', 'planparamsn'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
ACFG.online = 0;
ACFG.gamma = 0.95;
ACFG.planalg = 'opd';
ACFG.planparams.U = [-3 0 3];
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 50;
ACFG.reset = [-pi, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\opd_detip';
ACFG.X = 'coarse';  % results also available for "coarser"

cd(ACFG.datadir); 

% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'regret', 'exectime', 'depth'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end


%% Near-optimal solution: fuzzyQI -- run if not present

% REMARK: This file was run while fuzzyQI had the bug where all F values were zero if 'term=zero'. 
% Nevertheless this file is OK since it leaves 'term' to 'ignore'.

cfg = ACFG;
dummymodel = ipsetup_problem('model', cfg.model_params{:});
cfg.xgrids = symequidgrid(201, dummymodel.maxx);
cfg.ugrids = {cfg.planparams.U};
cfg.maxiter = 1000;
cfg.eps = 1e-5;
cfg.datafile = 'fzqi_ip_nearopt_Ts005';

% add near-optimal solution to default config if need to compute regret
if any(strcmp('nearopt', expids)) || (strcmp(mode, 'regret') && ~exist([cfg.datafile '.mat'], 'file')),
    cfg.run = 1;
    fuzzyqi(cfg);
end;
if strcmp(mode, 'regret'),      % construct optimal solution in format required to evaluate simple regret
    [approx, theta] = fuzzyqi(['approx datafile=' cfg.datafile]);
    ACFG.hstar = @(x) approx.h(approx, theta, x);
    ACFG.Qstar = @(x, u) approx.q(approx, theta, x, u); 
end;

%% Offline, uniform expansion criterion // F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'unifd';
bcfg.legendprefix = 'Uniform';

acfg = ACFG;
acfg.planparams.n = [50 100:100:1000];
acfg.planparams.criterion = 'unif';

execbatchexp(bcfg.idinfix);


%% Offline, regular expansion criterion // F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'opd';
bcfg.legendprefix = 'OPD';

acfg = ACFG;
acfg.planparams.n = [50 100:100:1000];

execbatchexp(bcfg.idinfix);


end