%% config
% clear all;
x0 = [-1; 1];

A1 = [.9 0; .1 .8]; A2 = [.9 .5; 0 .8];
mcfg.A = {A1, A2};
mcfg.Q = eye(2);
mcfg.maxx = [10; 10];
mcfg.dwelltime = 1;

cfg = struct;
cfg.problem = 'switchedlin_problem';
cfg.model_params = {mcfg};
cfg.gamma = .98;
cfg.maxsteps = 100;
cfg.planalg = 'opd';
cfg.planparams.U = [1 2];
cfg.planparams.n = 200;
cfg.verb = 3;

%% for debugging purposes, run also with a dwell time constraint of 2
cfg.reset = x0;
cfg.run = 1;
cfg.dwelltime = 5;
cfg.planalg = 'opddwelltime';
% cfg.planparams.dwelltime = cfg.dwelltime;
cfg.datafile = 'swlin_dwelltime1_constraineddwelltime_test';
genmpc(cfg);

h = genmpc(['plottrial datafile=' cfg.datafile]);
% h.R(1)
cost = 0; for i = 1:length(h.x), cost = cost + h.x(:, i)' * mcfg.Q * h.x(:, i); end;
cost

%% run OPD in RH allowing any initial mode
cfg.reset = x0;
cfg.run = 1;
cfg.datafile = 'swlin_dwelltime1_test';
cfg.dwelltime = 0;
genmpc(cfg);

h = genmpc(['plottrial datafile=' cfg.datafile]);
% h.R(1)
cost = 0; for i = 1:length(h.x), cost = cost + h.x(:, i)' * mcfg.Q * h.x(:, i); end;
cost

%% run OPD in RH with initial mode fixed
cfg.run = 1;
sigma0 = 2;
cfg.reset = mcfg.A{sigma0} * x0;
cfg.dwelltime = 0;
cfg.datafile = 'swlin_dwelltime1_sigma0=2test';
genmpc(cfg);

h = genmpc(['plottrial datafile=' cfg.datafile]);
cost = x0' * mcfg.Q * x0; for i = 1:length(h.x), cost = cost + h.x(:, i)' * mcfg.Q * h.x(:, i); end;
cost

%% run OPD asking just for the first sequence

model = switchedlin_problem('model', mcfg);
% call OPD directly and run it in fixed-depth mode, rather than fixed-budget mode
plancfg.depth = 70;             % this is about the limit of computational feasibility in this example
plancfg.actiontype = 'seq';     % return the whole action sequence
plancfg.U = cfg.planparams.U;
plancfg.gamma = cfg.gamma;
plancfg.verb = 3.5;             % show depths reached to get a feel of how algo is doing
[useq, xseq, planinfo, plandata] = opd(x0, model, plancfg);
% the bounds are found in planinfo.bounds, [lower upper]
planinfo.bounds
% according to the analysis, their gap should be at most 
gap = cfg.gamma ^ planinfo.expandeddepth / (1-cfg.gamma)
% but in fact the real gap is:
realgap = diff(planinfo.bounds)


