function [curreg]=batch_eval_alfa(testcfg)

%% Test common config

TESTCFG.problem = 'ip';
TESTCFG.test_type = 'ipoff';
TESTCFG.test_postfix = '';
TESTCFG.algorithms = {};
TESTCFG.nsimarray = [];
TESTCFG.osp_Svals = [1];
TESTCFG.ovsp_alfa = 1;

TESTCFG.onePlot = true; %whether to use subplot or separate plots
TESTCFG.avgRegretGrapth = true;
TESTCFG.showDepthGraph = false;
TESTCFG.regretPoints = 1;

testcfg = checkparams(testcfg, TESTCFG);

test_problem=testcfg.problem;
test_type  = testcfg.test_type;
test_postfix = testcfg.test_postfix;
algorithms = testcfg.algorithms;
nsimarray =  testcfg.nsimarray;
OSP_Svals = testcfg.osp_Svals;
OVSP_alfa = testcfg.ovsp_alfa;

onePlot = testcfg.onePlot; %whether to use subplot or separate plots
avgRegretGrapth = testcfg.avgRegretGrapth;
showDepthGraph = testcfg.showDepthGraph;
regretPoints = testcfg.regretPoints;

%% Test common config
if strcmp(test_type,'onl'),
    test_result_type = 'return';
else
    test_result_type = 'regret';
end;

switch test_problem,
    case 'ip',
        Uval = 2;   Uvalstring = ['_U' num2str(Uval)];    Uvalstring = strrep(Uvalstring,'.','_');
        probl_params = [test_problem test_type Uvalstring test_postfix '_'];
        ccfg = ip_common_config(Uval,'coarse');
    case 'hiv',
        probl_params = [test_problem test_type test_postfix '_'];
        ccfg = hiv_common_config();
end;

%% run eval
plotlinestyle = {'b-o', 'g-s', 'r-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'};
avgreg   = zeros(size(algorithms,2), size(nsimarray,2)); %array for storing average regret/return for each algo, for each 'n' from the testset
reg      = zeros(size(algorithms,2), size(nsimarray,2), 403); 
avgdepth = zeros(size(algorithms,2), size(nsimarray,2));

S=4;
for alfa=OVSP_alfa,
    for algo_i = 1:size(algorithms,2),
        figure(1);  close;
        algo_names{algo_i} = [algorithms{algo_i}{1} algorithms{algo_i}{2}];
        if strcmp(algorithms{algo_i}{2},'_v10'),
            algo_names{algo_i} = [algo_names{algo_i} '_alfa=0.' num2str(alfa*10)];
        end
        algo_names{algo_i} = strrep(algo_names{algo_i},'_','\_');
        for nsim=nsimarray,
            algo_name    = algorithms{algo_i}{1};
            algo_version = algorithms{algo_i}{2};
            nsim_i = nsimarray==nsim;
            
            algo_params = ['_nsim=' num2str(nsim)];
            switch algo_name
                case {'osp','usp'}
                    algo_params = [algo_params '_S=' num2str(S)];
                case 'ovsp'
%                     algo_params = [algo_params '_alfa=' num2str(ceil(alfa)*10-1)];
                    algo_params = [algo_params '_alfa=' num2str(alfa*10)];
                case 'ossp'
                    algo_params = [algo_params '_alfa=' num2str(alfa*10)];
                case 'ossp2'
                    algo_params = [algo_params '_alfa=' num2str(0.98*10)];
                otherwise
            end;
            
            algo_params = strrep(algo_params,'.','_');
            ccfg.datafile = [probl_params algo_name algo_version algo_params];
            
            switch test_type,
                case 'onl'
                    figure(1);
                    hist=genmpc(['plottrial addtocrt datafile=' ccfg.datafile]);
                    avgreg(algo_i, nsim_i)=hist.R(1); 
                otherwise
                    if avgRegretGrapth,
                        [avgreg(algo_i, nsim_i)] = genmpc_exp('regret', ccfg);
                    else
                        [avgreg(algo_i, nsim_i) reg(algo_i, nsim_i, :)] = genmpc_exp('regret', ccfg);
                        avgReg = 0;
                         for i=[198+[0:regretPoints-1] 206-[0:regretPoints-1]], avgReg = avgReg + reg(algo_i, nsim_i,i); end
                         avgreg(algo_i, nsim_i) = double(avgReg)/(2*regretPoints);
%                         avgreg(algo_i, nsim_i) = reg(algo_i, nsim_i, 206);
                    end;

                    if showDepthGraph,
                        load(ccfg.datafile);
                        avgK=0; 
                        for i=1:size(planinfoh,2), avgK = avgK + double(planinfoh(i).depth); end
                        avgK = double(avgK)/size(planinfoh,2);
                        avgdepth(algo_i, nsim_i) = avgK; %ccfg.gamma^double(avgK)/(1-ccfg.gamma);        
                    end;
            end;
        end;
    end;
    %plot the results
    if onePlot,
        figure(2);     
        hold off;
        subplot((size(OVSP_alfa,2)>1) + 1,ceil(size(OVSP_alfa,2)/2),find(alfa==OVSP_alfa));
    else
        figure;
    end
    for algo_i = 1:size(algorithms,2),
        hold on;
        if ~strcmp(algorithms{algo_i}{2},'_v10'),
            plot(nsimarray,avgreg(algo_i,:),plotlinestyle{algo_i});        
        end;
    end;
    if ~onePlot, legend(algo_names); end;
    title([test_problem ' ' test_result_type ', alfa=' num2str(alfa)]); 
    
    if showDepthGraph,
        if onePlot,
            figure(3); 
            subplot((size(OVSP_alfa,2)>1) + 1,ceil(size(OVSP_alfa,2)/2),find(alfa==OVSP_alfa));
        else
            figure;
        end
        for algo_i = 1:size(algorithms,2),
            hold on;
            plot(nsimarray,avgdepth(algo_i,:),plotlinestyle{algo_i});        
        end;
        if ~onePlot, legend(algo_names); end;
        title([test_problem ' onestate treedepth, alfa=' num2str(alfa)]); 
    end;
    
%     pause;
end;
if onePlot,
    figure(2);
    legend(algo_names);
    if showDepthGraph,
        figure(3);
        legend(algo_names);
    end
end

curreg = avgreg(algo_i,:);