%% Optimistic planning experiments for the inverted pendulum
function resarm_oplearncomparison(mode, expids, custombcfg, customacfg, customplanparams)

%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
% BCFG.idfields = {'alg', 'problem', 'planalg', 'planparamsn'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% Use Lex's implementation of the RARM to make sure we have the same settings
addpath('d:\Work\msc-bsc-phd-advising\lexdaniels_planning\thesis-mfiles\Systems\Resonating_Arm\');

% algorithm
ACFG = struct;
ACFG.problem = 'rasetup_problem';
ACFG.model_params = {'new', 'type=ra maxu=2', 'Ts=0.05'}; % mix, time, energy
ACFG.online = 1;
ACFG.gamma = 0.95;
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 25;
ACFG.reset = [-.85, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = pwd; %'c:\mfiles\approxrl\systems\invpend_setup\soop_ip';
ACFG.online = 1;

cd(ACFG.datadir); 

% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;
if nargin < 4, customplanparams = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            acfg.planparams = copyfields(customplanparams, acfg.planparams);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            else 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% OPD -- to double-check we get the same results as in paper
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'opd';
bcfg.legendprefix = 'OPD';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planparamsn'};

acfg = ACFG;
acfg.planalg = 'opd';       % ok only if we use the implementation variant by LEX; in this directory so it should use it
acfg.planparams.n = [25 50 75 100]; 
acfg.planparams.U = [-2 0 2];

execbatchexp(bcfg.idinfix);

%% SOOP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'soop';
bcfg.legendprefix = 'SOOP';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsalpha'};

acfg = ACFG;
acfg.planalg = 'sooplp';
acfg.planparams.method = 'soop';
acfg.planparams.n = [25 50 75 100] * 3;
acfg.planparams.dimselect = 'disc';
acfg.planparams.alpha = 0.1:0.1:0.9;

execbatchexp(bcfg.idinfix);

%% LP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'lp';
bcfg.legendprefix = 'LP';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamslipschitz'};

acfg = ACFG;
acfg.planalg = 'sooplp';
acfg.planparams.method = 'lp';
acfg.planparams.dimselect = 'lp';
acfg.planparams.n = [25 50 75 100] * 3;
acfg.planparams.lipschitz = 0.1:0.1:1.5;

execbatchexp(bcfg.idinfix);


%% HOLOP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'holop';
bcfg.legendprefix = 'HOLOP';
bcfg.in = 'planparamsn';
bcfg.startat = 1;
bcfg.repeat = 10;
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsK', 'rep'};

acfg = ACFG;
acfg.planalg = 'holop';
acfg.planparams.n = [25 50 75 100] * 3;
acfg.planparams.K = [2 3 4 5 10 15 20 25 30];
acfg.planparams.detact = 0;

execbatchexp(bcfg.idinfix);


end     % main function