X0 = [0.1534 2*0.1534 -0.1561 0]';

N = 10;
c = zeros(4, N);
rad = ones(4, N);
uind = ones(4, N);  % index just the one action
cfg.U = 0;          % one zero action only
cfg.mc_maxsteps = 250;
cfg.gamma = 0.99;

model = sw_problem('model');
model.Ts = 0.1;

% [eJ, J, hist] = sw_mc_rbfnearestpolicy(c, rad, uind, m, X0, cfg);

% precompute discounting vector, pre-initialize reward vector
disc = cumprod([1 cfg.gamma+zeros(1, cfg.mc_maxsteps)]);

% form explicit policy for ease of use
h = cfg.U(:, uind);
N0 = size(X0, 2); J = zeros(N0, 1);

% time
Ts = model.Ts; 
Tend = cfg.mc_maxsteps * Ts;
T = 0:Ts:Tend;
% ode settings
odeopt  = odeset('events', @sw_events);
% action has to be global
global U__ T__
for i = 1:N0,
    % initialize simulation for i-th initial state 
    X = nan(cfg.mc_maxsteps+1, model.p);
    U__ = nan(cfg.mc_maxsteps+1, 1);
    T__ = [];
    R = nan(cfg.mc_maxsteps+1, 1);
    % simulate
    timespan = T;       % make sure dynamics are called each sample time
    kstart = 1; koffset = 0;
    [t,x,te,ye,ie] = ode45(@sw_dynamics, timespan, X0(:, i), odeopt, model, c, rad, h); 
    tend = t(end);
    kend = floor(tend/Ts)+1;
    % resume simulation only as long as the timespan is not exhausted,
    % and the detected event was a swing leg impact (index 1) and not failure (index 2)
    while tend < Tend,
        X(kstart:kend, :) = x((kstart:kend) - koffset, :);
        % there was an impact, compute discontinuous state transfer
        x_afterimpact = sw_impact(x(end,:));
        % continue simulation
        firstsample = ceil(tend/Ts) * Ts;
        kstart = kend + 1; 
        if tend < firstsample,
            timespan = [tend firstsample:Ts:Tend];
            koffset = kstart - 2;
        else
            timespan = firstsample:Ts:Tend;
            koffset = kstart - 1;
        end;
        [t,x,te,ye,ie] = ode45(@sw_dynamics, timespan, x_afterimpact, odeopt, model, c, rad, h);
        tend = t(end);
        kend = floor(tend/Ts)+1;
    end;
    X(kstart:kend, :) = x((kstart:kend) - koffset, :);
    % pad with failure state at the next sample time,
    % if simulation ended with failure
%     if ~isempty(ie) && ie(1) == 2,
%         Kend = kend+1;
%         X(kend+1, :) = x(end, :);
%         fail = 1;
%     else
        Kend = kend;
        fail = 0;
%     end;

    % compute reward
    R(2:Kend) = model.ru * abs(U__(1:Kend-1));
    if fail,
       R(Kend) = R(Kend) + model.rfall;
    end;

    % return of x0 is inner product of discounting and rewards
    J(i) = disc(1:Kend-1) * R(2:Kend);
    
    hist.x = X(1:Kend, :)';
    hist.u = [U__(1:Kend - 1); NaN];
    hist.t = 0:Ts:(Kend-1)*Ts;
    hist.r = [0; R(2:Kend)];
    figure(10); clf;
    subplot(221); hold on; plot(hist.t, hist.x(1, :), 'k', hist.t, hist.x(3, :), 'b'); legend('\theta', '\theta''');
    subplot(222); hold on; plot(hist.t, hist.x(2, :), 'k', hist.t, hist.x(4, :), 'b'); legend('\phi', '\phi''');
    subplot(223); hold on; stairs(hist.t, hist.u, 'k'); legend('\tau');
    subplot(224); hold on; plot(hist.t, hist.r, 'k'); legend('r');
end;

% expected value is the mean
eJ = mean(J);

