function out = sw_problem(mode)
% Simple walker problem setup.
%   OUT = SW_PROBLEM(MODE)
% This function conforms to the specifications established by SAMPLE_PROBLEM.

% max range for state variables
xrange = [-pi/2 pi/2; -pi/2 pi/2; -1.5 1.5; -1.5 1.5];
% typical range for state variables during walking
wrange = [-.16 .16; -.33 .33; -.2 0; -.4 .1];
maxu = .8;
% test 1
% theta   = [-pi/2 -pi/6 -.16 -.1 0 .1 0.1534 pi/6 pi/2];
% phi     = [-pi/2 -pi/6 -.33 -.1 0 .1 0.3068 pi/6 pi/2];
% dtheta  = [-1.5  -.5 -.2 -0.1561 -.1 0 .5 1.5];
% dphi    = [-1.5  -.4 -.2  0 .1 .5 1.5];
% tau     = [-.8 -.4 0 .4 .8];
% test 3
% theta   = [-pi/2 -pi/6 -.16 -.1 0 .1 0.1534 pi/6 pi/2];
% phi     = [-pi/2 -pi/6 -.33 -.1 0 .1 0.3068 pi/6 pi/2];
% dtheta  = [-1.5  -.25 -0.1561 -.1 0 1.5];
% dphi    = [-1.5  -.4 -.2  0 .1 1.5];
% tau     = [-.8 -.2 0 .2 .8];

% test 4
theta   = [-pi/2 -pi/6 -.16 -.1 0 0.1534+[-.003:.0015:.003] .2 .3 pi/6 pi/2];
phi     = [-pi/2 -pi/6 -.33 -.1 0 .1 0.3068 pi/6 pi/2];
dtheta  = [-1.5  -.25 -0.1561+[-.003:.0015:.003] -.1 0 .5 1.5];
dphi    = [-1.5  -.4 -.2  0 .1 1.5];
tau     = [-.8 -.2 0 .2 .8];

xgrids  = {theta, phi, dtheta, dphi};
ugrids  = {tau};

% --------- Initial state
% autonomous walking (on the limit cycle)
x0 = [0.1534 2*0.1534 -0.1561 0];
% init randomly in typical walking range
% x0      = wrange(:, 1) + rand(4, 1) .* diff(wrange, 1, 2);

switch mode
    case 'model'        % since the model is simple, we keep everything top-level
        % physics
        model.Ts        = 0.03;          % simulation step size
        model.slope     = 0.004;        % slope angle
        model.p         = 4;
        model.q         = 1;
        % bounds
        model.xrange    = xrange;
        model.maxu      = maxu;
        % functions
        model.fun       = 'sw_mdp';
        model.plotfun   = 'sw_animate';
        % rewards
        model.rfall     = -10;
        model.ru        = -5;
        
        out = model;
                
    case 'fuzzy'
        % quantization
        cfg.xgrids = xgrids;
        cfg.ugrids = ugrids;
        % learning
        cfg.gamma   = .99;
        cfg.maxiter = 300;
        cfg.eps     = 0.1;
        % replay
        cfg.tend    = 20;
        cfg.x0      = x0;
        
        out = cfg;
       
end;


% END sw_problem(), RETURNING out ===========================================