
%% IP Setup tests
cfg = struct;

% cfg.problem = 'hillcar_problem';
% cfg.U = {[-4 4]};
% cfg.datafile = 'hillcar_fittedqi_test';

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'lqr', 'type=ip maxu=3', 'Ts=0.03'};
cfg.U = {[-3 0 3]};
cfg.datafile = 'ipsetup_fittedqi_test';

% cfg.problem = 'ipsetup_problem';
% cfg.model_params = {'lqr', 'type=nowrap maxu=3', 'Ts=0.03'};
% cfg.U = {[-3 0 3]};
% cfg.datafile = 'ipsetupnowrap_fittedqi_test';

cfg.gamma = .98;        % discount factor
cfg.regmethod = 'extratrees';
cfg.maxiter = 50;       % max # iterations: make it quick
cfg.trees_ntrees = 40;    % # of trees for ensemble methods
% cfg.trees_k = [];       % k param for extra trees; defaults to #of regression vars
cfg.trees_nmin = 2;       % nmin param for extra trees
cfg.datadir = pwd;
cfg.samples = 'method=randdisc N=10000';
cfg.verb = 3;
% cfg.storehistory = 1;

% Neural Fitted QI test
cfg.gamma = .98;          % discount factor
cfg.regmethod = 'nn';
cfg.maxiter = 20;         % max # iterations: make it quick
cfg.datadir = pwd;
cfg.samples = 'method=randdisc N=30000';
cfg.nn_sizes = [5 1];
cfg.nn_tfs = {'tansig', 'purelin'};
cfg.nn_trainfun = 'trainlm';
cfg.Xtest = {-pi:pi/6:pi, -10*pi:pi:10*pi};
cfg.eps = 0.1;
cfg.nn_epochs = 250;
cfg.verb = 4;
cfg.storehistory = 1;

% run
rcfg = cfg;
rcfg.run = 1;
fittedqi(rcfg);

% % replay
% pcfg = cfg;
% pcfg.replay = 1;
% pcfg.x0 = [-pi; 0];
% fittedqi(pcfg);     % meaningless results -- need to check plots...

% fittedqi('sol datafile=fitqi_test');
% fittedqi('evol datafile=fitqi_test');

%% 2010-02, DC motor tests with and without single-tree mode

ccfg = struct;
ccfg.problem = 'dc_problem';
ccfg.model_params = {'lqr', 'lin'};
ccfg.gamma = .95;
ccfg.datadir = 'd:\work\m\approxrl\testing\test_fittedqi\';

fcfg = ccfg;
fcfg.run = 1;
fcfg.regmethod = 'extratrees';
fcfg.singlereg = 0;        % use one regression tree for each discrete action
fcfg.maxiter = 50;         % max # iterations
fcfg.trees_ntrees = 50;    % # of trees for ensemble methods
fcfg.trees_k = [];         % k param for extra trees; [] = defaults to #of regression vars
fcfg.trees_nmin = 2;       % nmin param for extra trees
% use a grid of samples/action discretization
fcfg.samples.method = 'grid';
fcfg.samples.xgrids = [50 50];
fcfg.samples.ugrids = {[-10 0 10]};
% output config
fcfg.itersave = 10;
fcfg.verb = 3;
fcfg.U = {[-10 0 10]};
    
fcfg.datafile = 'dc_fittedqi_test_multiplereg';
fittedqi(fcfg);

fcfg.singlereg = 1;
fcfg.datafile = 'dc_fittedqi_test_singlereg';
fittedqi(fcfg);



%% 2010-04, DC motor tests with and without convergence test

ccfg = struct;
ccfg.problem = 'dc_problem';
ccfg.model_params = {'lqr', 'lin'};
ccfg.gamma = .95;
ccfg.datadir = 'd:\work\m\approxrl\testing\test_fittedqi\';
%
ccfg.run = 1;
ccfg.regmethod = 'extratrees';
ccfg.singlereg = 0;        % use one regression tree for each discrete action
ccfg.maxiter = 50;         % max # iterations
ccfg.trees_ntrees = 50;    % # of trees for ensemble methods
ccfg.trees_k = [];         % k param for extra trees; [] = defaults to #of regression vars
ccfg.trees_nmin = 2;       % nmin param for extra trees
% use a grid of samples/action discretization
ccfg.samples.method = 'grid';
ccfg.samples.xgrids = [50 50];
ccfg.samples.ugrids = {[-10 0 10]};
% output config
ccfg.itersave = 10;
ccfg.verb = 3;
ccfg.U = {[-10 0 10]};
    
%% no conv test
fcfg = ccfg;
fcfg.datafile = 'dc_fittedqi_test_noconvtest';
fittedqi(fcfg);

%% conv test enabled, using all data samples
fcfg = ccfg;
fcfg.Xtest = 'auto';
fcfg.datafile = 'dc_fittedqi_test_convtest';
fittedqi(fcfg);

%% no conv test, single-regressor
fcfg = ccfg;
fcfg.singlereg = 1;
fcfg.datafile = 'dc_fittedqi_test_singlereg_noconvtest';
fittedqi(fcfg);

%% conv test enabled, using all data samples
fcfg = ccfg;
fcfg.singlereg = 1;
fcfg.Xtest = 'auto';
fcfg.datafile = 'dc_fittedqi_test_singlereg_convtest';
fittedqi(fcfg);
    
