%% Initialization

% common config
ccfg.target = 'screen';      % set to "files" to output figures and movies; to "screen" for live demo
% ccfg.target = 'files';      % set to "files" to output figures and movies; to "screen" for live demo
ccfg.problem = 'ipsetup_problem';
ccfg.gamma = 0.98;
ccfg.model_params = {'rewtype=nlqr Qrew=[1,0] Rrew=0.1', 'type=ip maxx=[pi;15*pi] maxu=2', 'Ts=0.025'};
ccfg.eps = 1;
ccfg.datadir = pwd;
% visualizer config
ccfg.viscfg = struct;
ccfg.viscfg.figsize = [800 500];
ccfg.viscfg.pause = -1;


%% 4) Optimistic planning

% algorithm parameters
cfg = ccfg;
% we also change the reward function to so-called "normalized LQR" -- the same as before, except now
% it's scaled and translated into [0,1]
cfg.model_params = {'nlqr', 'type=ip maxx=[pi;15*pi] maxu=3', 'Ts=0.05'};
% planning algorithm parameters
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.ntrans = 2000;         % budget of expansions at each step
cfg.planparams.U = [-2 0 2];    % action space -- the same as for M=3 in the approximators above
cfg.reset = [pi/2, 0]';          % always reset pointing down
cfg.trials = 1;                 % one trial is enough to see performance, as we're not learning
cfg.maxsteps = 1;              % 80 steps = 4 seconds should be enough for a swingup
% output config
cfg.datadir = pwd;
cfg.datafile = 'ipsetup_optplan';
% use visualization -- set visualize to 0 to disable
% cfg.visualize = 3;             % this means: only visualize system steps
% cfg.itervis = 1;
% cfg.viscfg.pause = 0.01;      
cfg.saveplandata = 1;
cfg.actiontype = 'seq';

% run the algorithm
cfg.run = 1;
genmpc(cfg);              


%% 
close all;
% df = load('iponl3_U2_opd_n=2000=24000_S=3.mat');
df = load('ipsetup_optplan');
gamma = 0.98;
root = 1;

[maxd child] = max(df.plandatah{1}.TR)
Tparent = df.plandatah{1}.Tparent;
Tdepth = df.plandatah{1}.Tdepth;
TR = df.plandatah{1}.TR;

i=1;
maxpath = child;
while child~=root,
    nu(i) = TR(child);
    B(i) = nu(i) + gamma^double(Tdepth(child)) / (1-gamma);
    child = Tparent(1,child);
    maxpath = [child maxpath];
    i = i + 1;
end

plot(1:i-1, nu);%, 1:i-1, B);

