%% 1. DOUBLEINT: common init
% model and representative initial states
mcfg = struct;
mcfg.matype = 'fixedconn'; mcfg.rewtype = 'none'; 
mcfg.type = 'doubleint'; mcfg.dim = 1; mcfg.Ts = 0.5;
mcfg.range = 5; 
% mcfg.flock_dist = 5; mcfg.flock_maxdvel = 5;

% mcfg.nagents = 4;
% P = [.5 .25 .25 0;
%     .25 .5 0 .25;
%     .25 0 .5 .25;
%     0 .25 .25 .5];
% prange = 5;
% vrange = 5;
% x0 = [(2*rand(1, mcfg.nagents) - 1).*prange; -3 -1 1 3];
% % x0 = [(2*rand(1, mcfg.nagents) - 1).*prange; (2*rand(1, mcfg.nagents) - 1).*vrange];
% % x0 = [-10 0 10; -2 3 -2];

mcfg.nagents = 5;
mcfg.G = [0 1 1 0 0;
          1 0 1 0 0;
          1 1 0 1 1;
          0 0 1 0 1;
          0 0 1 1 0];
alphaP = 0.2; P = mcfg.G .* alphaP; for i = 1:mcfg.nagents, P(i, i) = 1-sum(P(i, :)); end;
x0 = [0 1 2 3 4; -3 -1 0 1 3];

m = malinear_problem('model', mcfg);
Atilde = m.A^2;
Btilde = [m.A*m.B m.B];
Mu = norm(pinv(Btilde)) * (1+norm(Atilde, 2)) * norm(x0(:), inf);
alphabar = norm(Btilde);

% algorithms
acfg = struct;
acfg.problem = 'malinear_problem';
acfg.model_params = {mcfg};

acfg.K = 2;
acfg.P = P; 
acfg.optparams.maxu = repmat(Mu, acfg.K, 1);
acfg.trials = 1;
acfg.maxsteps = 40;
% visualization
acfg.verb = 3;
acfg.datadir = 'd:\work\m\approxrl\testing\test_maopd';
acfg.reset = x0(:);      % reset state to these values
acfg.visualize = 2;
acfg.viscfg.pause = .01;

%% DOUBLEINT: DOO
cfg = acfg;
acfg.optparams.method = 'doo';
acfg.optparams.diam = @(urange) alphabar * norm((urange(2, :) - urange(1, :))./2, 2);
acfg.optparams.n = 100;
acfg.datafile = 'oocons_doubleint_doo';
acfg.run = 1; ooconsensus(acfg);

%% DOUBLEINT: SOO
cfg = acfg;
acfg.optparams.method = 'soo';
acfg.optparams.dmax = @(t) sqrt(t);
acfg.optparams.n = 100;
acfg.datafile = 'oocons_doubleint_soo';
acfg.run = 1; ooconsensus(acfg);



%% 2. Robot arm
macfg = struct; 
macfg.nagents = 3; macfg.G = [0 1 1; 1 0 0; 1 0 0]; % undirected graph 2 -- 1 -- 3
macfg.maxomega = [15*pi;  15*pi];          % [rad/sec] maximum angular speeds of links
macfg.maxtau =   [50;    50];            % [Nm] maximum torques
macfg.orientation = 'horizontal';
macfg.Ts = 0.05;

P = [0.5 0.25 0.25; 
     0.25 0.75 0; 
     0.25 0 0.75];  % compatible with graph
% macfg.Q = diag([1 .1 1 .1]);

cfg = struct;
cfg.problem = 'mararm_problem';
cfg.model_params = {macfg}; 
cfg.optparams.method = 'doo';
cfg.optparams.diam = @(urange) 3*norm((urange(2, :) - urange(1, :))./2, 2);  % for DOO, assuming locally linear and Lipschitz ct taken 1
cfg.optparams.dmax = @(t) sqrt(t); % for SOO
cfg.optparams.n = 300;
cfg.optparams.verb = 1;
cfg.trials = 1;
cfg.maxsteps = 100;
cfg.K = 2;
cfg.P = P;
cfg.optparams.maxu = repmat(macfg.maxtau, cfg.K, 1); % repeat bounds
% regular initial states
% cfg.reset = [0, 0, 0, 0; pi/2, 0, 0, 0; -pi/2, 0, 0, 0]'; cfg.datafile = 'oocons_mararm_G2-1-3_soo';
% random initial angles, arms at rest
cfg.reset = 'randangles'; cfg.datafile = 'oocons_mararm_G2-1-3_doolipschitz3_randangles'; 
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.run = 1;
ooconsensus(cfg);


%% 3. Inverted pendulum
macfg = struct; 
macfg.nagents = 3; macfg.maxu = 20; 
macfg.G = [0 1 1; 1 0 0; 1 0 0]; % undirected graph 2 -- 1 -- 3
macfg.Ts = 0.05;

P = [0.5 0.25 0.25; 
     0.25 0.75 0; 
     0.25 0 0.75];  % compatible with graph

cfg = struct;
cfg.problem = 'mainvpend_problem';
cfg.model_params = {macfg}; 
cfg.optparams.method = 'doo';
cfg.optparams.diam = @(urange) 3*norm((urange(2, :) - urange(1, :))./2, 2);  % for DOO, assuming locally linear
% cfg.optparams.dmax = @(t) sqrt(t); % for SOO
cfg.optparams.n = 500;
cfg.optparams.verb = 1;
cfg.trials = 1;
cfg.maxsteps = 100;
cfg.K = 2;
cfg.P = P;
cfg.optparams.maxu = repmat(macfg.maxu, cfg.K, 1); % repeat bounds
% regular initial states
% cfg.reset = [0, 0; pi/2, 0; -pi/2, 0]'; cfg.datafile = 'oocons_mainvpend_G2-1-3_soo';
% random initial angles, arms at rest
cfg.reset = 'randangles'; cfg.datafile = 'oocons_mainvpend__G2-1-3_doolipschitz3_randangles'; 
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.run = 1;
ooconsensus(cfg);