%% init also requires this
clear ccfg mcfg rcfg;
crtdir = pwd;
ccfg = struct;
ccfg.gamma = 0.9;
ccfg.problem = 'dc_problem';
ccfg.datadir = [pwd '/'];

mcfg.type = 'lin';
% mcfg.maxx = [pi; pi*15]; mcfg.maxu = 30;     % sufficiently large
% tighter bounds to decrease the confidence intervals
mcfg.maxx = [pi; pi*5]; mcfg.maxu = 10;     % sufficiently large
mcfg.Ts = .01;
% variant 1: gaussian noise (uncountable next-states); OLOP required
% mcfg.noise = 'gauss'; mcfg.gausssigma = .01 * eye(2); id = 'gauss';
% variant 2: finite next states
mcfg.stoch_prob = [.7 .3]; mcfg.stoch_gain = [1 0.5]; id = 'finite';
rcfg.rewtype = 'nlqr'; rcfg.Q = diag([1 .001]); rcfg.R = .01;
ccfg.model_params = {rcfg, mcfg};

% plot configs
% ccfg.discU = [-10 -3 0 3 10];      % discrete action space
ccfg.discU = [-5 0 5];      % discrete action space
% ccfg.restrictx = [2*pi/3 2*pi];
ccfg.x0 = [-pi; 0];
ccfg.tend = 1;
ccfg.plottarget = 'fig';
ccfg.savedir = [pwd '/'];
ccfg.npoints = 100;
ccfg.gridres = 100;
ccfg.figsize = [400 240];           % default fig size
ccfg.trajfigsize = [460 500];       % default trajfigsize
ccfg.shortfigsize = [400 150];           % default fig size
% ccfg.dctrajylim = [-.1 0.8*pi; -10*pi 2*pi; -15 5; 0.5 1];

%% OLOP experiment run

acfg = ccfg;
acfg.online = 1;
acfg.actiontype = 'seq';
acfg.maxseqlen = Inf; 
acfg.maxseqlen = 0.5;
% plan params
acfg.planalg = 'olop';
acfg.planparams.n = 10000;
acfg.planparams.U = ccfg.discU;
acfg.planparams.cucb = 1;
acfg.planparams.verb = 1;
acfg.planparams.compat = 0;
acfg.trials = 1;
acfg.reset = ccfg.x0;
acfg.maxsteps = ccfg.tend/mcfg.Ts;
acfg.verb = 2;
acfg.datafile = sprintf('dc%s_olop_n%d_maxseqlen%s_cUCB%s', ...
    id, acfg.planparams.n, num2strx(acfg.maxseqlen, 'r'), num2strx(acfg.planparams.cucb, 'r'));

acfg.run = 1;
genmpc(acfg);

% trajectory
cfg = ccfg;
% cfg.planalg = 'olop';
cfg.datafile = acfg.datafile;
cfg.savefig = [cfg.datafile '_traj'];
cfg.plottrial = 1;
cfg.figsize = ccfg.trajfigsize;
% cfg.ylim = ccfg.dctrajylim;
hist = genmpc(cfg);
Rolop = hist.R(1)

%% OPSS experiment run: only discrete noise supported

acfg = ccfg;
acfg.online = 1;
acfg.actiontype = 'seq';
acfg.maxseqlen = Inf; 
% plan params
acfg.planalg = 'opss';
acfg.planparams.n = 100; %round(5000/6);
acfg.planparams.L = length(mcfg.stoch_gain);
acfg.planparams.U = ccfg.discU;
acfg.planparams.verb = 2;
acfg.planparams.compat = 0;
acfg.trials = 1;
acfg.reset = ccfg.x0;
acfg.maxsteps = ccfg.tend/mcfg.Ts;
acfg.verb = 3;
acfg.datafile = sprintf('dc%s_opss_n%d', id, acfg.planparams.n);

acfg.run = 1;
genmpc(acfg);

% trajectory
cfg = ccfg;
cfg.datafile = acfg.datafile;
cfg.savefig = [cfg.datafile '_traj'];
cfg.plottrial = 1;
cfg.figsize = ccfg.trajfigsize;
hist = genmpc(cfg);
Ropss = hist.R(1)
