%% Test policy tree (variant that supports subtrees, 11 june 2013)
cfg.problem = 'ipsetup_problem';
m = ipsetup_problem('model', 'nlqr', 'type=ip maxu=3 stoch_prob=[.95,.05] stoch_gain=[1,.7]', 'Ts=0.05');
discount = 0.95;
U = [-3 0 3];
L = 2;

%% test random nodes
plancfg = struct;
% plancfg.n = 1000;
plancfg.depth = 5;
plancfg.gamma = discount;
plancfg.L = L;
plancfg.U = U;
plancfg.verb = 2;
plancfg.actiontype = 'seq';

x0 = [-pi; 0];

[H0, pinfo0, T0] = opss(x0, m, plancfg);

for i = 1:10,
    node = unidrnd(H0.size);
    [H1, pinfo1, T1] = opss(H0.Hx(:, node), m, plancfg);
    fprintf('nu(H0(x''))=%.3f, nu(H1(x'')) = %.3f\n', H0.Hutil(node), H1.Hutil(1)); 
    if H1.Hutil(1) < H0.Hutil(node), error('Smaller nu on replanned policy'); end;
end;

%% subpolicy, test smallest-diameter node
plancfg = struct;
% plancfg.n = 1000;
plancfg.depth = 7;
plancfg.depthprime = 4;
plancfg.gamma = discount;
plancfg.L = L;
plancfg.U = U;
plancfg.verb = 2;
plancfg.actiontype = 'seq';

x0 = [-pi; 0];

[H0, pinfo0, T0] = opss(x0, m, plancfg);

leaves = find(H0.Hleaf);
diams = T0.Tb(H0.Ht(leaves)) - H0.Hutil(leaves);
[mindiam imin] = min(diams); node = leaves(imin);
[H1, pinfo1, T1] = opss(H0.Hx(:, node), m, plancfg);
fprintf('min diam: nu(H0(x''))=%.3f, nu(H1(x'')) = %.3f\n', H0.Hutil(node), H1.Hutil(1)); 
if H1.Hutil(1) < H0.Hutil(node), error('Smaller nu on replanned policy'); end;

[maxcontrib imax] = min(H0.Hc(leaves)); node = leaves(imax);
[H1, pinfo1, T1] = opss(H0.Hx(:, node), m, plancfg);
fprintf('max contrib: nu(H0(x''))=%.3f, nu(H1(x'')) = %.3f\n', H0.Hutil(node), H1.Hutil(1)); 
if H1.Hutil(1) < H0.Hutil(node), error('Smaller nu on replanned policy'); end;

%% test random nodes, strict node inclusion
plancfg = struct;
% plancfg.n = 1000;
plancfg.depth = 5;
plancfg.gamma = discount;
plancfg.L = L;
plancfg.U = U;
plancfg.verb = 2;
plancfg.actiontype = 'seq';

x0 = [-pi; 0];

[H0, pinfo0, T0] = opss(x0, m, plancfg);

for i = 1:10,
    
    node = unidrnd(H0.size);
  
   % find all nodes in subtree of node
    frontier = node; subtree = node;
    while ~all(H0.Hleaf(frontier)),            % there are still inner nodes to process
        newnodes = [];
        for node2 = frontier(~H0.Hleaf(frontier)),     % loop through each inner node
            % add children on greedy branch, accounting for the possibility of fewer than L children
            children = H0.Hchildren(:, node2)'; children = children(children>0);
            newnodes = [newnodes children];
        end;
        % new set of nodes is old set of LEAF nodes, plus the newly added nodes
        frontier = [frontier(H0.Hleaf(frontier)) newnodes];
        subtree = [subtree newnodes];
    end;
    
    [H1, pinfo1, T1] = opss(H0.Hx(:, node), m, plancfg);
    
    for node2 = subtree,
        if isempty(findflat(H0.Hx(:, node2), T1.Tx)), 
            error('state in H0(x'') not found on T1(x'')'); 
        end;
    end;
    fprintf('Subtree of node %d checked\n', node);
end;

