close all;  clear all;
Uval = 2;
Uvals = '_U2';
ccfg = ip_common_config(Uval);
[approx, theta] = fuzzyqi(['approx datafile=ip_fzqi_nearopt' Uvals]);
ccfg.hstar = @(x) approx.h(approx, theta, x);
ccfg.Qstar = @(x, u) approx.q(approx, theta, x, u); 


%% IP offline tests for a set of n, K=5
narray = [600:300:4500 6000 7500];
K = 16;
testParamNames = [Uvals '_K=16'];

for nsim=narray,
    %offline OPD
    cfg = ccfg;
    cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
    cfg.planparams.n = floor(nsim/size(cfg.planparams.U,2));         % budget of expansions at each step
    cfg.datafile = ['ipoff_opd' testParamNames '_nsim=' num2str(nsim)];
    cfg.online = 0;
    cfg.run = 1;
    genmpc(cfg);              
    %offline okp exploiting redundancy
    cfg = ccfg;
    cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
    cfg.planparams.n = floor(nsim/K/size(cfg.planparams.U,2));         % budget of expansions at each step
    cfg.planparams.K = K;
    cfg.planparams.epsil = 10^(-10);
    cfg.planparams.exploitredundancy = true;
    cfg.datafile = ['ipoff_okp' testParamNames '_nsim=' num2str(nsim)];   
    cfg.online = 0;
    cfg.run = 1;
    genmpc(cfg);  

    %offline okp exploiting redundancy equal treesize
    cfg = ccfg;
    cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
    cfg.planparams.nsim = nsim;       % budget of expansions *3 since there are 3 discrete actions
    cfg.planparams.K = K;
    cfg.planparams.epsil = 10^(-10);
    cfg.planparams.exploitredundancy = true;
    cfg.datafile = ['ipoff_okp_eqTree' testParamNames '_nsim=' num2str(nsim)];   
    cfg.online = 0;
    cfg.run = 1;
    genmpc(cfg);      
end;


%% IP offline tests for a set of K, n=640
n = 640;
%offline OPD
cfg = ccfg;
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = n;         % budget of expansions at each step
cfg.datafile = ['ipoff_opd_n=' num2str(n)];
cfg.online = 0;
cfg.run = 1;
genmpc(cfg);              

Karray = [1 2 4 5 8 10 16 20 32];
for K=Karray,
    testParamNames = [Uvals '_K=' num2str(K)];
    %offline okp exploiting redundancy
    cfg = ccfg;
    cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
    cfg.planparams.n = n/K;         % budget of expansions at each step
    cfg.planparams.K = K;
    cfg.planparams.epsil = 10^(-10);
    cfg.planparams.exploitredundancy = true;
    cfg.datafile = ['ipoff_okp' num2str(n) 'K=' num2str(K)];   
    cfg.online = 0;
    cfg.run = 1;
    genmpc(cfg);    

    %offline okp exploiting redundancy
    cfg = ccfg;
    cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
    cfg.planparams.nsim = n*size(cfg.planparams.U,2);         % budget of expansions at each step
    cfg.planparams.K = K;
    cfg.planparams.epsil = 10^(-10);
    cfg.planparams.exploitredundancy = true;
    cfg.datafile = ['ipoff_okp_eqTree' num2str(n) 'K=' num2str(K)];   
    cfg.online = 0;
    cfg.run = 1;
    genmpc(cfg);      
end;